using System;
using System.Data;
using System.Data.Common;
using System.Configuration;
using System.IO;

namespace Vevo.Shared.DataAccess
{
    /// <summary>
    /// Summary description for DataAccessHelper
    /// </summary>
    public static class DataAccessHelper
    {
        #region Private

        private static bool _enable = true;

        private static DbProviderFactory _dbFactory;

        private static string _dbConnectionString;
        private static string _dbProviderName;
        private static bool _showVerboseError;
        private static bool _cacheEnabled;
        private static DataAccess.LogErrorCallback _logError;
        private static string _urlCultureName;
        private static string _domainRegistrationKey;
        private static string _domainName;
        private static string _licenseFilePath;
        private static string _secretKey;

        private static int _decimalPlaceNumber = 2;

        private static string _urlCultureID;
        private static string _liteCultureID = "1";


        private static string GetFullPathNameFromDBFileName( string relativeFileName, string appLocalPath )
        {
            relativeFileName = relativeFileName.Replace( '/', '\\' ); // replace / with \
            if (relativeFileName.StartsWith( "~\\" ))
                relativeFileName = relativeFileName.Substring( 2 );
            else if (relativeFileName.StartsWith( "\\" ))
                relativeFileName = relativeFileName.Substring( 1 );

            return Path.Combine( appLocalPath, relativeFileName );
        }

        #endregion


        #region Public Properties

        public static string DomainRegistrationkey
        {
            get
            {
                return _domainRegistrationKey;
            }
            set
            {
                _domainRegistrationKey = value;
            }
        }

        public static string DomainName
        {
            get
            {
                return _domainName;
            }
            set
            {
                _domainName = value;
            }
        }

        public static string LicenseFilePath
        {
            get
            {
                return _licenseFilePath;
            }
        }

        public static string DbConnectionString
        {
            get
            {
                if (_enable)
                    return _dbConnectionString;
                else
                {
                    _enable = true;
                    return "";
                }
            }
        }

        public static string DbProviderName
        {
            get
            {
                if (_enable)
                    return _dbProviderName;
                else
                {
                    _enable = true;
                    return "";
                }
            }
        }

        public static bool ShowVerboseError
        {
            get
            {
                return _showVerboseError;
            }
            set
            {
                _showVerboseError = value;
            }
        }

        public static bool CacheEnabled
        {
            get
            {
                return _cacheEnabled;
            }
            set
            {
                _cacheEnabled = value;
            }
        }

        public static DataAccess.LogErrorCallback LogError
        {
            get
            {
                return _logError;
            }
            set
            {
                _logError = value;
            }
        }

        public static void DisableConnectionOneTime()
        {
            _enable = false;
        }

        public static string UrlCultureID
        {
            get
            {
                if (_urlCultureID == null &&
                    _urlCultureName != null)
                {
                    //_urlCultureID = CultureAccess.GetIDByName( _urlCultureName );
                    _urlCultureID = "1";        // Use English culture (en-US) as default
                }

                return _urlCultureID;
            }
            set
            {
                _urlCultureID = value;
            }
        }

        public static int DecimalPlaceNumber
        {
            get
            {
                return _decimalPlaceNumber;
            }
            set
            {
                _decimalPlaceNumber = value;
            }
        }

        public static string SecretKey
        {
            get { return _secretKey; }
        }

        public static string LiteCultureID
        {
            get
            {
                return _liteCultureID;
            }
            set
            {
                _liteCultureID = value;
            }
        }

        #endregion


        #region Public Methods

        public static string CreateConnectionStringFromPrefix(
            string dbConnectionStringPrefix,
            string dbConnectionString,
            string dbProviderName,
            string appLocalPath )
        {
            string result;

            if (String.Compare( dbProviderName, "System.Data.OleDb", true ) == 0)
            {
                result =
                    dbConnectionStringPrefix +
                    GetFullPathNameFromDBFileName( dbConnectionString, appLocalPath );
            }
            else
            {
                result = dbConnectionString;
            }

            return result;
        }

        public static void SetUp(
            string dbConnectionString,
            string dbProviderName,
            bool showVerboseError,
            DataAccess.LogErrorCallback logError,
            string secretKey )
        {
            _dbFactory = null;

            _dbConnectionString = dbConnectionString;
            _dbProviderName = dbProviderName;
            _showVerboseError = showVerboseError;
            _logError = logError;
            _secretKey = secretKey;
            _urlCultureID = null;
        }

        public static void SetUp(
            string dbConnectionString,
            string dbProviderName,
            bool showVerboseError,
            bool cacheEnabled,
            DataAccess.LogErrorCallback logError,
            string secretKey,
            string urlCultureName,
            string licenseFilePath )
        {
            _dbFactory = null;

            _dbConnectionString = dbConnectionString;
            _dbProviderName = dbProviderName;
            _logError = logError;
            _showVerboseError = showVerboseError;
            _cacheEnabled = cacheEnabled;
            _urlCultureName = urlCultureName;
            _secretKey = secretKey;
            _urlCultureID = null;
            _licenseFilePath = licenseFilePath;
        }

        public static DbProviderFactory GetProviderFactory()
        {
            if (_dbFactory == null)
                _dbFactory = DbProviderFactories.GetFactory( DataAccessHelper.DbProviderName );

            return _dbFactory;
        }

        public static DbConnection CreateConnection()
        {
            DbConnection conn = GetProviderFactory().CreateConnection();
            conn.ConnectionString = DataAccessHelper.DbConnectionString;
            return conn;
        }

        public static bool IsMicrosoftAccess()
        {
            if (DbConnectionString.ToLower().Contains( ".mdb" ) &&
                String.Compare( DbProviderName, "System.Data.OleDb", true ) == 0)
                return true;
            else
                return false;
        }

        #endregion

    }
}
