using System;
using System.Collections.Generic;
using System.Net.Mail;
using System.Text;
using Vevo.Shared.Domain.SystemServices;

namespace Vevo.Shared.Domain.SystemServices
{
    // This class implement interfaces IMailSendService
    // The derived classes only need to define the mechanism to actually send email
    public abstract class BaseMailSendService : IMailSendService
    {
        #region Private

        private MailMessage CreateMailMessage(
            MailAddress from, string to, string subject, string body )
        {
            MailMessage mailMessage = new MailMessage();

            string[] toList = to.Split( new char[] { ';', ',' } );
            foreach (string recipient in toList)
            {
                if (!String.IsNullOrEmpty( recipient.Trim() ))
                    mailMessage.To.Add( new MailAddress( recipient ) );
            }
            mailMessage.From = from;
            mailMessage.Subject = subject;
            mailMessage.Body = body;

            return mailMessage;
        }

        private MailMessage CreateHtmlMailMessage(
            MailAddress from, string to, string subject, string body )
        {
            MailMessage mailMessage = CreateMailMessage( from, to, subject, body );
            mailMessage.IsBodyHtml = true;
            mailMessage.BodyEncoding = System.Text.Encoding.UTF8;
            mailMessage.SubjectEncoding = System.Text.Encoding.UTF8;

            return mailMessage;
        }

        #endregion


        #region Protected

        protected abstract void SendMail( MailMessage mailMessage );

        #endregion


        #region IMailSendService Members

        public void SendMail( string from, string to, string subject, string body )
        {
            SendMail(
                new MailAddress( from ),
                to,
                subject,
                body );
        }

        public void SendMail( MailAddress from, string to, string subject, string body )
        {
            MailMessage mailMessage = CreateMailMessage( from, to, subject, body );

            SendMail( mailMessage );
        }

        public void SendHtmlMail( string from, string to, string subject, string body )
        {
            SendHtmlMail(
                new MailAddress( from ),
                to,
                subject,
                body );
        }

        public void SendHtmlMail( MailAddress from, string to, string subject, string body )
        {
            MailMessage mailMessage = CreateHtmlMailMessage( from, to, subject, body );

            SendMail( mailMessage );
        }

        #endregion
    }
}
