using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

namespace Vevo.Shared.WebUI
{
    public class UrlPath
    {
        #region Private
        private static string _storefrontUrl;

        private string _url;


        private string RemoveDoubleSlash( string url )
        {
            return url.Replace( "//", "/" );
        }

        private string RemoveQueryString( string url )
        {
            int index = url.IndexOf( '?' );
            if (index == -1)
                return url;
            else
                return url.Substring( 0, index );
        }

        private string RemovePage( string urlNoQuery )
        {
            int index = urlNoQuery.LastIndexOf( '/' );
            if (index == -1)
                return urlNoQuery;
            else
                return urlNoQuery.Substring( 0, index );
        }

        private string ExtractLastFolder( string urlFolderOnly )
        {
            int index = urlFolderOnly.LastIndexOf( '/' );
            if (index == -1)
                return urlFolderOnly;
            else
                return urlFolderOnly.Substring( index + 1 );
        }

        private static int GetHostIndex( string url )
        {
            int hostIndex = url.IndexOf( "://" );
            if (hostIndex != -1)
                hostIndex += 3;

            return hostIndex;
        }

        private static string GetHostUrl()
        {
            string url = HttpContext.Current.Request.Url.AbsoluteUri;

            int endIndex = url.IndexOf( '/', GetHostIndex( url ) );

            return url.Substring( 0, endIndex );
        }

        private int SkipTildeAndSlash()
        {
            int index = 0;
            while (index < _url.Length)
            {
                if (_url[index] != '/' && _url[index] != '~')
                    return index;
            }

            return -1;
        }

        private int SkipHost()
        {
            int hostIndex = GetHostIndex( _url );
            if (hostIndex == -1)
                return 0;

            return _url.IndexOf( '/', hostIndex );
        }

        private int SkipApplicationPath( int startIndex )
        {
            if (startIndex == -1)
                return -1;

            string tildePath = "~/";
            if (_url.Substring( startIndex ).StartsWith( tildePath ))
                return startIndex + tildePath.Length;

            string applicationPath = HttpContext.Current.Request.ApplicationPath.ToLower();

            if (_url.ToLower().Substring( startIndex ).StartsWith( applicationPath ))
            {
                int index = startIndex + applicationPath.Length;
                if (_url[index] == '/')
                    return index + 1;
                else
                    return index;
            }
            else
            {
                return startIndex;
            }
        }

        private string ExtractOneFolder( int startIndex )
        {
            if (startIndex == -1)
                return String.Empty;

            int endIndex = _url.IndexOf( '/', startIndex );
            if (endIndex == -1)
                return String.Empty;
            else
                return _url.Substring( startIndex, endIndex - startIndex );
        }
        #endregion


        #region Constructors

        public UrlPath( string url )
        {
            _url = url;
        }

        #endregion


        #region Public Properties

        // Returns url of the root of web application, e.g. "http://www.mydomain.com/".
        // If the current page is not a root path, this function will return the root path.
        // For example, if the current page is http://www.mydomain.com/Admin/Default.aspx, 
        // this function will return "http://www.mydomain.com/".
        //
        // However, if the root application is a virtual directory, this function will also
        // return that directory. For example, if "http://www.myotherweb.com/store/" is the root
        // path of the store website, this function will return this value.
        //
        // This function will ends the path with slash (/).
        public static string StorefrontUrl
        {
            get
            {
                //if (_storefrontUrl == null)
                //{
                    _storefrontUrl = GetHostUrl() + HttpContext.Current.Request.ApplicationPath;
                    if (_storefrontUrl[_storefrontUrl.Length - 1] != '/')
                        _storefrontUrl += "/";
                //}

                return _storefrontUrl;
            }
        }

        #endregion


        #region Public Methods

        public static string GetDisplayedUrl()
        {
            return GetHostUrl() + HttpContext.Current.Request.RawUrl;
        }

        public static string ExtractWWWandHTTPinUrl( string url )
        {
            if (url.Substring( 0, "http://".Length ).ToLower().Contains( "http://" ))
            {
                url = url.Substring( "http://".Length );
            }
            else if (url.Substring( 0, "https://".Length ).ToLower().Contains( "https://" ))
            {
                url = url.Substring( "https://".Length );
            }

            if (url.Substring( 0, "www.".Length ).ToLower().Contains( "www." ))
            {
                url = url.Substring( "www.".Length );
            }

            if (url.Substring( url.Length - 1, 1 ).ToLower().Contains( "/" ))
            {
                url = url.Substring( 0, url.Length - 1 );
            }

            return url;
        }

        public static string GetStoreFrontUrlWithoutWWWandHTTP()
        {
            return ExtractWWWandHTTPinUrl( StorefrontUrl );
        }

        public bool IsSslEnabled()
        {
            return _url.ToLower().StartsWith( "https://" );
        }

        public bool IsStorefrontUrl()
        {
            return String.IsNullOrEmpty( ExtractFirstApplicationSubfolder() );
        }

        public string ExtractFirstApplicationSubfolder()
        {
            if (String.IsNullOrEmpty( _url ))
                return String.Empty;

            int index = SkipHost();
            index = SkipApplicationPath( index );
            return ExtractOneFolder( index );
        }

        public string ExtractRelativePathFromApplicationFolder()
        {
            if (String.IsNullOrEmpty( _url ))
                return String.Empty;

            int index = SkipHost();
            index = SkipApplicationPath( index );
            return _url.Substring( index );
        }

        public string CreateSslUrl()
        {
            return _url.Replace( "http://", "https://" );
        }

        public string CreateAbsoluteUri()
        {
            string path = _url.Substring( SkipHost() );

            return StorefrontUrl + path.TrimStart( '~', '/' );
        }

        public string CreateAbsoluteUri( string urlStorefront )
        {
            string path = _url.Substring( SkipHost() );

            if (urlStorefront[urlStorefront.Length - 1] != '/')
                urlStorefront += "/";

            return urlStorefront + path.TrimStart( '~', '/' );
        }

        public bool IsAffiliateUrl()
        {
            return (IsStorefrontUrl() && RemoveQueryString( _url ).ToLower().Contains( "/affiliate" ));
        }

        #endregion

    }
}
