﻿using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Base.Domain;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Blogs;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class BlogCategoryRepository : IBlogCategoryRepository
    {
        #region Private

        private const string RepositoryName = " BlogCategoryRepository";

        private const string GetLocalesSignature = "GetLocales";
        private const string GetOneSignature = "GetOne";
        private const string GetBlogCategoryIDFromBlogCategoryNameSignature = "GetBlogCategoryIDFromBlogCategoryName";
        private const string GetBlogCategoryIDFromBlogCategoryURLSignature = "GetBlogCategoryIDFromBlogCategoryURL";
        private const string GetBlogCategoryNameFromBlogCategoryURLSignature = "GetBlogCategoryNameFromBlogCategoryURL";
        private const string GetAllSignature = "GetAll";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchBlogCategorySignature = "SearchBlogCategory";

        private DataCache _blogCategoryDataCache = new DataCache(RepositoryName);
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility(new string[] { "BlogCategoryLocale", "BlogCategory" });

        private BlogCategoryRepositoryNoCache _repository = new BlogCategoryRepositoryNoCache();
        #endregion

       #region Constructors

        public BlogCategoryRepository()
        {
        }

        public BlogCategoryRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods
        public BlogCategory Save( BlogCategory blogcategory )
        {
            BlogCategory newBlogCategory = _repository.Save( blogcategory );
            _blogCategoryDataCache.ClearRepositoryCache();
            return newBlogCategory;
        }

        public void Delete( string blogcategoryID )
        {
            _repository.Delete( blogcategoryID );
            _blogCategoryDataCache.ClearRepositoryCache();
        }

        public void DeleteLocales( string blogcategoryID )
        {
            _repository.DeleteLocales( blogcategoryID );
            _blogCategoryDataCache.ClearRepositoryCache();
        }

        public IList<ILocale> GetLocales( string blogcategoryID )
        {
            IList<ILocale> list = (IList<ILocale>) _blogCategoryDataCache.GetValue(
                 GetLocalesSignature,
                 _cacheDependency,
                 blogcategoryID );

            if (list == null)
            {
                list = _repository.GetLocales( blogcategoryID );

                _blogCategoryDataCache.SetValue(
                    GetLocalesSignature,
                    list,
                    blogcategoryID );
            }

            return list;
        }

        public void DeleteAll()
        {
            _repository.DeleteAll();
            _blogCategoryDataCache.ClearRepositoryCache();
        }

        public BlogCategory GetOne( Culture culture, string blogcategoryID )
        {
            BlogCategory blogcategory = (BlogCategory) _blogCategoryDataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, blogcategoryID );

            if (blogcategory == null)
            {
                blogcategory = _repository.GetOne( culture, blogcategoryID );
                _blogCategoryDataCache.SetValue( GetOneSignature, blogcategory, culture.CultureID, blogcategoryID );
            }

            return blogcategory;
        }

        public string GetBlogCategoryIDFromBlogCategoryName( string blogcategoryName )
        {
            string blogcategoryID = (string) _blogCategoryDataCache.GetValue( GetBlogCategoryIDFromBlogCategoryNameSignature, _cacheDependency, blogcategoryName );

            if ((blogcategoryID == string.Empty) || (blogcategoryID == null))
            {
                blogcategoryID = _repository.GetBlogCategoryIDFromBlogCategoryName( blogcategoryName );
                _blogCategoryDataCache.SetValue( GetBlogCategoryIDFromBlogCategoryNameSignature, blogcategoryName );
            }

            return blogcategoryID;
        }

        public string GetBlogCategoryIDFromBlogCategoryURL(string blogCategoryName)
        {
            string blogcategoryID = (string)_blogCategoryDataCache.GetValue(GetBlogCategoryIDFromBlogCategoryURLSignature, _cacheDependency, blogCategoryName);

            if ((blogcategoryID == string.Empty) || (blogcategoryID == null))
            {
                blogcategoryID = _repository.GetBlogCategoryIDFromBlogCategoryURL(blogCategoryName);
                _blogCategoryDataCache.SetValue(GetBlogCategoryIDFromBlogCategoryURLSignature, blogCategoryName);
            }

            return blogcategoryID;
        }

        public string GetBlogCategoryNameFromBlogCategoryURL(string blogCategoryURL)
        {
            string blogcategoryName = (string)_blogCategoryDataCache.GetValue(GetBlogCategoryNameFromBlogCategoryURLSignature, _cacheDependency, blogCategoryURL);

            if ((blogcategoryName == string.Empty) || (blogcategoryName == null))
            {
                blogcategoryName = _repository.GetBlogCategoryNameFromBlogCategoryURL(blogCategoryURL);
                _blogCategoryDataCache.SetValue(GetBlogCategoryNameFromBlogCategoryURLSignature, blogCategoryURL);
            }

            return blogcategoryName;
        }

        public IList<BlogCategory> GetAll( Culture culture, BoolFilter isEnabled, string sortBy )
        {
            IList<BlogCategory> list = (IList<BlogCategory>) _blogCategoryDataCache.GetValue(
                 GetAllSignature,
                 _cacheDependency,
                 culture.CultureID, isEnabled, sortBy );

            if (list == null)
            {
                list = _repository.GetAll( culture, isEnabled, sortBy );

                _blogCategoryDataCache.SetValue(
                    GetAllSignature,
                    list,
                    culture.CultureID, isEnabled, sortBy );
            }

            return list;
        }

        public IList<BlogCategory> GetAll(
            Culture culture,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            Pair list = (Pair) _blogCategoryDataCache.GetValue(
                GetAllSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );

            if (list == null)
            {
                list = new Pair();

                list.First = _repository.GetAll(
                    culture,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex,
                    out howManyItems );
                list.Second = howManyItems;

                _blogCategoryDataCache.SetValue(
                    GetAllSignature,
                    list,
                    culture.CultureID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<BlogCategory>) list.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _blogCategoryDataCache.GetValue( GetTableSchemaSignature, _cacheDependency );
            if (list == null)
            {
                list = _repository.GetTableSchema();
                _blogCategoryDataCache.SetValue( GetTableSchemaSignature, list );
            }
            return list;
        }

        public IList<BlogCategory> SearchBlogCategory( Culture culture, string sortBy, SearchFilter searchFilter,
            BoolFilter isActive, out int howManyItems )
        {
            Pair list = (Pair) _blogCategoryDataCache.GetValue( SearchBlogCategorySignature,
                _cacheDependency,
                culture.CultureID, sortBy, searchFilter, isActive );

            if (list == null)
            {
                list = new Pair();
                list.First = _repository.SearchBlogCategory(
                    culture, sortBy, searchFilter, isActive, out howManyItems );
                list.Second = howManyItems;
                _blogCategoryDataCache.SetValue(
                    SearchBlogCategorySignature,
                    list,
                    culture.CultureID, sortBy, searchFilter, isActive );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<BlogCategory>) list.First;
        }

        public void UpdateSortOrder( string[] sortedBlogCategoryIDs )
        {
            _repository.UpdateSortOrder( sortedBlogCategoryIDs );
            _blogCategoryDataCache.ClearRepositoryCache();
        }

        public BlogCategory GetOneByUrlName( Culture culture, string urlName )
        {
            BlogCategory blogcategory = (BlogCategory) _blogCategoryDataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, urlName );

            if (blogcategory == null)
            {
                blogcategory = _repository.GetOneByUrlName( culture, urlName );
                _blogCategoryDataCache.SetValue( GetOneSignature, blogcategory, culture.CultureID, urlName );
            }

            return blogcategory;
        }

        #endregion
    }
}
