using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Payments;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class GiftCertificateRepositoryNoCache : IGiftCertificateRepository
    {
        #region Private
        private const string AllFields = " GiftCertificateCode, ProductID, OrderItemID, GiftValue, " +
            " RemainValue, UrlParameter, Recipient, PersonalNote, IsExpirable, ExpireDate, NeedPhysical, IsActive ";

        private GiftCertificate CreateGiftCertificate( GiftCertificate giftCertificate )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO GiftCertificate " +
                "(" + AllFields + ") " +
                "VALUES (@GiftCertificateCode, @ProductID, @OrderItemID, @GiftValue, @RemainValue, @UrlParameter, " +
                "@Recipient, @PersonalNote, @IsExpirable, @ExpireDate, @NeedPhysical, @IsActive); ",
                DataAccess.CreateParameterString( giftCertificate.GiftCertificateCode ),
                DataAccess.CreateParameterString( giftCertificate.ProductID ),
                DataAccess.CreateParameterString( giftCertificate.OrderItemID ),
                DataAccess.CreateParameterDecimal( giftCertificate.GiftValue ),
                DataAccess.CreateParameterDecimal( giftCertificate.RemainValue ),
                DataAccess.CreateParameterString( giftCertificate.UrlParameter ),
                DataAccess.CreateParameterString( giftCertificate.Recipient ),
                DataAccess.CreateParameterString( giftCertificate.PersonalNote ),
                DataAccess.CreateParameterBool( giftCertificate.IsExpirable ),
                DataAccess.CreateParameterDateTime( giftCertificate.ExpireDate ),
                DataAccess.CreateParameterBool( giftCertificate.NeedPhysical ),
                DataAccess.CreateParameterBool( giftCertificate.IsActive )
                );

            return giftCertificate;
        }

        private GiftCertificate UpdateGiftCertificate( GiftCertificate giftCertificate )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE GiftCertificate " +
                "SET GiftValue = @GiftValue, " +
                "RemainValue = @RemainValue, " +
                "Recipient = @Recipient, " +
                "PersonalNote = @PersonalNote, " +
                "IsExpirable = @IsExpirable, " +
                "ExpireDate = @ExpireDate, " +
                "NeedPhysical = @NeedPhysical, " +
                "IsActive = @IsActive " +
                "WHERE GiftCertificateCode = @GiftCertificateCode",
                DataAccess.CreateParameterDecimal( giftCertificate.GiftValue ),
                DataAccess.CreateParameterDecimal( giftCertificate.RemainValue ),
                DataAccess.CreateParameterString( giftCertificate.Recipient ),
                DataAccess.CreateParameterString( giftCertificate.PersonalNote ),
                DataAccess.CreateParameterBool( giftCertificate.IsExpirable ),
                DataAccess.CreateParameterDateTime( giftCertificate.ExpireDate ),
                DataAccess.CreateParameterBool( giftCertificate.NeedPhysical ),
                DataAccess.CreateParameterBool( giftCertificate.IsActive ),
                DataAccess.CreateParameterString( giftCertificate.GiftCertificateCode )
                );

            return giftCertificate;
        }

        private GiftCertificate.ExpirationStatus GetExpirationStatus( DataRow row )
        {
            GiftCertificate.ExpirationStatus status = GiftCertificate.ExpirationStatus.NotExpired;

            if (ConvertUtilities.ToBoolean( row["IsExpirable"] ))
            {
                if (DateTime.Today > ConvertUtilities.ToDateTime( row["ExpireDate"] ))
                    status = GiftCertificate.ExpirationStatus.Expired;
            }
            else if (ConvertUtilities.ToDecimal( row["RemainValue"] ) == 0)
                status = GiftCertificate.ExpirationStatus.NoValue;
            else if (!ConvertUtilities.ToBoolean( row["IsActive"] ))
                status = GiftCertificate.ExpirationStatus.NotActive;

            return status;
        }

        private GiftCertificate CreateOneFromRow( DataRow row )
        {
            GiftCertificate giftCertificate = new GiftCertificate();
            giftCertificate.GiftCertificateCode = row["GiftCertificateCode"].ToString();
            giftCertificate.ProductID = row["ProductID"].ToString();
            giftCertificate.OrderItemID = row["OrderItemID"].ToString();
            giftCertificate.GiftValue = (decimal) row["GiftValue"];
            giftCertificate.RemainValue = (decimal) row["RemainValue"];
            giftCertificate.UrlParameter = row["UrlParameter"].ToString();
            giftCertificate.Recipient = row["Recipient"].ToString();
            giftCertificate.PersonalNote = row["PersonalNote"].ToString();
            giftCertificate.IsExpirable = (bool) row["IsExpirable"];
            giftCertificate.ExpireDate = (DateTime) row["ExpireDate"];
            giftCertificate.NeedPhysical = (bool) row["NeedPhysical"];
            giftCertificate.IsActive = (bool) row["IsActive"];
            giftCertificate.ExpireStatus = GetExpirationStatus( row );

            return giftCertificate;
        }

        private IList<GiftCertificate> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<GiftCertificate> list = new List<GiftCertificate>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "productid":
                    return "GiftCertificate.ProductID";
                case "orderitemid":
                    return "GiftCertificate.OrderItemID";
                case "giftcertificatecode":
                    return "GiftCertificate.GiftcertificateCode";
                case "orderid":
                    return "Orders.OrderID";

                default:
                    return fieldName;
            }
        }

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "giftcertificatecode" ))
                return "ORDER BY GiftCertificate." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "productid" ))
                return "ORDER BY GiftCertificate." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower() == "orderitemid")
                return "ORDER BY GiftCertificate." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower() == "orderid")
                return "ORDER BY OrderItem." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy;
        }

        #endregion

        #region Public Methods

        public GiftCertificate Create( GiftCertificate giftCertificate )
        {
            return CreateGiftCertificate( giftCertificate );
        }

        public GiftCertificate Update( GiftCertificate giftCertificate )
        {
            return UpdateGiftCertificate( giftCertificate );
        }

        public GiftCertificate GetOne( string giftCertificateCode )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM GiftCertificate " +
                "WHERE GiftCertificateCode = @GiftCertificateCode;",
                DataAccess.CreateParameterString( giftCertificateCode )
            );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return GiftCertificate.Null;
        }

        public string GetCodeFromUrlParameter( string urlParameter )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM GiftCertificate " +
                "WHERE UrlParameter = @UrlParameter;",
                DataAccess.CreateParameterString( urlParameter )
            );

            if (table.Rows.Count > 0)
                return table.Rows[0]["GiftCertificateCode"].ToString();
            else
                return String.Empty;
        }

        public IList<GiftCertificate> GetAllByOrderID( string orderID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GiftCertificateCode, GiftCertificate.ProductID, GiftCertificate.OrderItemID, GiftValue, RemainValue, " +
                " UrlParameter, Recipient, PersonalNote, IsExpirable, ExpireDate, NeedPhysical, Name, IsActive " +
                "FROM GiftCertificate LEFT JOIN OrderItem ON OrderItem.OrderItemID = GiftCertificate.OrderItemID " +
                "WHERE OrderID = @OrderID " +
                "ORDER BY GiftCertificate.OrderItemID;",
                DataAccess.CreateParameterString( orderID ) );
            if (table.Rows.Count > 0)
                return ConvertTableToList( table, 0, table.Rows.Count - 1 );
            else
                return new List<GiftCertificate>();
        }

        public string GetCodeFromOrderItemID( string orderItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT GiftCertificateCode " +
                "FROM GiftCertificate " +
                "WHERE OrderItemID = @OrderItemID;",
                DataAccess.CreateParameterString( orderItemID )
            );

            if (table.Rows.Count > 0)
                return table.Rows[0]["GiftCertificateCode"].ToString();
            else
                return String.Empty;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT GiftCertificate.GiftCertificateCode, GiftCertificate.ProductID, GiftCertificate.OrderItemID," +
                 " GiftValue, RemainValue, Recipient, PersonalNote, IsExpirable, " +
                 " ExpireDate, NeedPhysical, Name, IsActive, UserName, OrderItem.OrderID, CustomerID " +
                 "FROM (Orders INNER JOIN OrderItem ON Orders.OrderID = OrderItem.OrderID) " +
                 " INNER JOIN GiftCertificate ON OrderItem.OrderItemID = GiftCertificate.OrderItemID " +
                 "WHERE 0 = 1; "
            );

            DataColumn[] keys = new DataColumn[] { table.Columns["GiftCertificateCode"] };
            table.PrimaryKey = keys;
            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public void Delete( string giftCertificateCode )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GiftCertificate " +
                "WHERE GiftCertificateCode = @GiftCertificateCode ;",
                DataAccess.CreateParameterString( giftCertificateCode ) );
        }

        public void DeleteByOrderID( string orderID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GiftCertificate " +
                "WHERE OrderItemID in (SELECT OrderItemID from OrderItem WHERE OrderID = @OrderID);",
                DataAccess.CreateParameterString( orderID ) );
        }

        public void DeleteByOrderItemID( string orderItemID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM GiftCertificate " +
                "WHERE OrderItemID = @OrderItemID;",
                DataAccess.CreateParameterString( orderItemID ) );
        }

        public DataTable SearchGiftCertificate(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            string giftFieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, giftFieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT GiftCertificate.GiftCertificateCode, GiftCertificate.ProductID, GiftCertificate.OrderItemID," +
                " GiftValue, RemainValue, UrlParameter, Recipient, PersonalNote, IsExpirable, " +
                " ExpireDate, NeedPhysical, Name, IsActive, UserName, OrderItem.OrderID, CustomerID " +
                "FROM (Orders INNER JOIN OrderItem ON Orders.OrderID = OrderItem.OrderID) " +
                " INNER JOIN GiftCertificate ON OrderItem.OrderItemID = GiftCertificate.OrderItemID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[1] { full.Columns["GiftCertificateCode"] };
            full.PrimaryKey = keys;

            return DataAccess.CopyTableRows( full, startIndex, endIndex );
        }

        #endregion
    }
}
