using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;
using Vevo.Shared.DataAccess;
using Vevo.Shared.DataAccess.Implementation;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;
using Vevo.Base.Data;
using Vevo.Deluxe.Domain;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain.CustomerReward;

namespace Vevo.Data
{
    public class OrderRepositoryNoCache : IOrderRepository
    {
        //GatewayPaymentStatus	GatewayOrderID
        #region Private
        private const string _allFieldNotEncryption =
            " OrderID, OrderDate, CustomerID, UserName, FirstName, LastName, Company, Address1, " +
            "Address2, City, State, Zip, Country, Phone, Fax, Email, ShippingFirstName, ShippingLastName, " +
            "ShippingCompany, ShippingAddress1, ShippingAddress2, ShippingCity, ShippingState, ShippingZip, " +
            "ShippingCountry, ShippingPhone, ShippingFax, ShippingResidential, " +
            "CustomerComments, PaymentMethod, ShippingMethod, " +
            "Status, IPAddress, Subtotal, Tax, ShippingCost, CouponID, CouponDiscount, BaseCurrencyCode, " +
            "UserCurrencyCode, UserConversionRate, InvoiceNotes, ShippingID, " +
            "GiftCertificateCode, GiftCertificate, " +
            "TrackingNumber, TrackingMethod, GiftRegistryID, ShowShippingAddress, HandlingFee, " +
            "QbPostDate, QbReferenceCode, " +
            "ContainsRecurring, ParentID, PaymentToken, " +
            "AvsAddrStatus, AvsZipStatus, CvvStatus, StoreID, QBTxnID, IsQBExported, PONumber, " +
            "IsTaxExempt, TaxExemptID, TaxExemptCountry, TaxExemptState, " +
            "RedeemPoint, RedeemPrice, PointEarned, IsPointApplied, IsSubscriptionApplied ";

        private Order Create( Order order )
        {
            string orderID = DataAccess.ExecuteScalar(
                "INSERT INTO Orders ( CustomerID, UserName, FirstName, LastName, Address1, Address2, City, " +
                "State, Zip, Country, Phone, Fax, Email, ShippingFirstName, ShippingLastName, ShippingCompany, " +
                "ShippingAddress1, ShippingAddress2, ShippingCity, ShippingState, ShippingZip, ShippingCountry, " +
                "ShippingPhone, ShippingFax, ShippingResidential, " +
                "CustomerComments, PaymentMethod, GatewayPaymentStatus, GatewayOrderID, " +
                "ShippingMethod, PaymentComplete, Status, IPAddress, Subtotal, Tax, ShippingCost, CouponID, CouponDiscount, " +
                "BaseCurrencyCode, UserCurrencyCode, UserConversionRate, InvoiceNotes, ShippingID, GiftCertificateCode, " +
                "GiftCertificate, TrackingNumber, TrackingMethod, GiftRegistryID, ShowShippingAddress, HandlingFee, " +
                "ContainsRecurring, ParentID, PaymentToken, AvsAddrStatus, AvsZipStatus, CvvStatus, StoreID, isQBExported, PONumber, " +
                "IsTaxExempt, TaxExemptID, TaxExemptCountry, TaxExemptState, " +
                "RedeemPoint, RedeemPrice, PointEarned, IsPointApplied, IsSubscriptionApplied  ) " +
                "VALUES ( @CustomerID, @UserName, @FirstName, @LastName, @Address1, @Address2, @City, @State, " +
                "@Zip, @Country, @Phone, @Fax, @Email, @ShippingFirstName, @ShippingLastName, " +
                "@ShippingCompany, @ShippingAddress1, @ShippingAddress2, " +
                "@ShippingCity, @ShippingState,@ShippingZip, @ShippingCountry, " +
                "@ShippingPhone, @ShippingFax, @ShippingResidential, " +
                "@CustomerComments, @PaymentMethod, @GatewayPaymentStatus, @GatewayOrderID, " +
                "@ShippingMethod, @PaymentComplete, @Status, @IPAddress, @Subtotal, @Tax, @ShippingCost, @CouponID, @CouponDiscount, " +
                "@BaseCurrencyCode, @UserCurrencyCode, @UserConversionRate, @InvoiceNotes, @ShippingID, " +
                "@GiftCertificateCode, @GiftCertificate, @TrackingNumber, @TrackingMethod, @GiftRegistryID, " +
                "@ShowShippingAddress, @HandlingFee, @ContainsRecurring, @ParentID, @PaymentToken, " +
                "@AvsAddrStatus, @AvsZipStatus, @CvvStatus , @StoreID, @isQBExported, @PONumber, " +
                "@IsTaxExempt, @TaxExemptID, @TaxExemptCountry, @TaxExemptState, " +
                "@RedeemPoint, @RedeemPrice, @PointEarned, @IsPointApplied, @IsSubscriptionApplied  ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( order.CustomerID ),
                DataAccess.CreateParameterString( order.UserName ),
                DataAccess.CreateParameterString( order.Billing.FirstName ),
                DataAccess.CreateParameterString( order.Billing.LastName ),
                DataAccess.CreateParameterString( order.Billing.Address1 ),
                DataAccess.CreateParameterString( order.Billing.Address2 ),
                DataAccess.CreateParameterString( order.Billing.City ),
                DataAccess.CreateParameterString( order.Billing.State ),
                DataAccess.CreateParameterString( order.Billing.Zip ),
                DataAccess.CreateParameterString( order.Billing.Country ),
                DataAccess.CreateParameterString( order.Billing.Phone ),
                DataAccess.CreateParameterString( order.Billing.Fax ),
                DataAccess.CreateParameterString( order.Email ),
                DataAccess.CreateParameterString( order.Shipping.FirstName ),
                DataAccess.CreateParameterString( order.Shipping.LastName ),
                DataAccess.CreateParameterString( order.Shipping.Company ),
                DataAccess.CreateParameterString( order.Shipping.Address1 ),
                DataAccess.CreateParameterString( order.Shipping.Address2 ),
                DataAccess.CreateParameterString( order.Shipping.City ),
                DataAccess.CreateParameterString( order.Shipping.State ),
                DataAccess.CreateParameterString( order.Shipping.Zip ),
                DataAccess.CreateParameterString( order.Shipping.Country ),
                DataAccess.CreateParameterString( order.Shipping.Phone ),
                DataAccess.CreateParameterString( order.Shipping.Fax ),
                DataAccess.CreateParameterBool( order.Shipping.Residential ),
                DataAccess.CreateParameterString( order.CustomerComments ),
                DataAccess.CreateParameterString( order.PaymentMethod ),
                DataAccess.CreateParameterString( order.GatewayPaymentStatus ),
                DataAccess.CreateParameterString( order.GatewayOrderID ),
                DataAccess.CreateParameterString( order.ShippingMethod ),
                DataAccess.CreateParameterBool( order.PaymentComplete ),
                DataAccess.CreateParameterString( order.Status ),
                DataAccess.CreateParameterString( order.IPAddress ),
                DataAccess.CreateParameterDecimalMoney( order.Subtotal ),
                DataAccess.CreateParameterDecimalMoney( order.Tax ),
                DataAccess.CreateParameterDecimalMoney( order.ShippingCost ),
                DataAccess.CreateParameterString( order.CouponID ),
                DataAccess.CreateParameterDecimalMoney( order.CouponDiscount ),
                DataAccess.CreateParameterString( order.BaseCurrencyCode ),
                DataAccess.CreateParameterString( order.UserCurrencyCode ),
                DataAccess.CreateParameterDouble( order.UserConversionRate ),
                DataAccess.CreateParameterString( order.InvoiceNotes ),
                DataAccess.CreateParameterString( order.ShippingID ),
                DataAccess.CreateParameterString( order.GiftCertificateCode ),
                DataAccess.CreateParameterDecimal( order.GiftCertificate ),
                DataAccess.CreateParameterString( order.TrackingNumber ),
                DataAccess.CreateParameterString( order.TrackingMethod ),
                DataAccess.CreateParameterString( ConvertUtilities.ToInt32( order.GiftRegistryID ).ToString() ),
                DataAccess.CreateParameterBool( order.ShowShippingAddress ),
                DataAccess.CreateParameterDecimalMoney( order.HandlingFee ),
                DataAccess.CreateParameterBool( order.ContainsRecurring ),
                DataAccess.CreateParameterString( order.ParentID ),
                DataAccess.CreateParameterString( order.PaymentToken ),
                DataAccess.CreateParameterString( order.AvsAddrStatus ),
                DataAccess.CreateParameterString( order.AvsZipStatus ),
                DataAccess.CreateParameterString( order.CvvStatus ),
                DataAccess.CreateParameterString( order.StoreID ),
                DataAccess.CreateParameterBool( false ),
                DataAccess.CreateParameterString( order.PONumber ),
                DataAccess.CreateParameterBool( order.IsTaxExempt ),
                DataAccess.CreateParameterString( order.TaxExemptID ),
                DataAccess.CreateParameterString( order.TaxExemptCountry ),
                DataAccess.CreateParameterString( order.TaxExemptState ),
                DataAccess.CreateParameterInt32( order.RedeemPoint ),
                DataAccess.CreateParameterDecimalMoney( order.RedeemPrice ),
                DataAccess.CreateParameterInt32( order.PointEarned ),
                DataAccess.CreateParameterBool( order.IsPointApplied ),
                DataAccess.CreateParameterBool( order.IsSubscriptionApplied )
                );

            order.OrderID = orderID;

            return order;
        }

        private Order Update( Order order )
        {
            string sql = "UPDATE Orders " +
                "SET CustomerID = @CustomerID, OrderDate = @OrderDate, UserName = @UserName, " +
                "    FirstName = @FirstName, LastName = @LastName, Company = @Company, " +
                "    Address1 = @Address1, Address2 = @Address2, City = @City, State = @State, " +
                "    Zip = @Zip, Country = @Country, Phone = @Phone, Fax = @Fax, Email = @Email, " +
                "    ShippingFirstName = @ShippingFirstName, ShippingLastName = @ShippingLastName, " +
                "    ShippingCompany = @ShippingCompany, ShippingAddress1 = @ShippingAddress1, " +
                "    ShippingAddress2 = @ShippingAddress2, ShippingCity = @ShippingCity, " +
                "    ShippingState = @ShippingState, ShippingZip = @ShippingZip, " +
                "    ShippingCountry = @ShippingCountry, ShippingPhone = @ShippingPhone, " +
                "    ShippingFax = @ShippingFax, ShippingResidential = @ShippingResidential, " +
                "    PaymentMethod = @PaymentMethod, GatewayPaymentStatus = @GatewayPaymentStatus, " +
                "	 GatewayOrderID = @GatewayOrderID, ShippingMethod = @ShippingMethod, " +
                "    PaymentComplete = @PaymentComplete, " +
                "    Processed = @Processed, Status = @Status, Cancelled = @Cancelled, " +
                "    IPAddress = @IPAddress, Subtotal = @Subtotal, Tax = @Tax, ShippingCost = @ShippingCost, " +
                "    CouponID = @CouponID, CouponDiscount = @CouponDiscount, " +
                "    CustomerComments = @CustomerComments, " +
                "    BaseCurrencyCode = @BaseCurrencyCode, UserCurrencyCode = @UserCurrencyCode, " +
                "    UserConversionRate = @UserConversionRate, InvoiceNotes = @InvoiceNotes, " +
                "    GiftCertificateCode = @GiftCertificateCode, GiftCertificate = @GiftCertificate," +
                "    TrackingNumber = @TrackingNumber, TrackingMethod = @TrackingMethod, " +
                "    HandlingFee = @HandlingFee, " +
                "    ContainsRecurring = @ContainsRecurring, " +
                "    AvsAddrStatus = @AvsAddrStatus, AvsZipStatus = @AvsZipStatus, CvvStatus = @CvvStatus , " +
                "    StoreID = @StoreID, QBTxnID = @QBTxnID, IsQBExported = @IsQBExported, PONumber = @PONumber, " +
                "    IsTaxExempt = @IsTaxExempt, TaxExemptID = @TaxExemptID, TaxExemptCountry = @TaxExemptCountry, " +
                "    TaxExemptState = @TaxExemptState, RedeemPoint = @RedeemPoint, " +
                "    RedeemPrice = @RedeemPrice, PointEarned = @PointEarned, IsPointApplied = @IsPointApplied,  " +
                "    IsSubscriptionApplied = @IsSubscriptionApplied " +
                " WHERE OrderID = @OrderID; ";

            DataAccess.ExecuteNonQuery( sql
                ,
                DataAccess.CreateParameterString( order.CustomerID ),
                DataAccess.CreateParameterDateTime( order.OrderDate ),
                DataAccess.CreateParameterString( order.UserName ),
                DataAccess.CreateParameterString( order.Billing.FirstName ),
                DataAccess.CreateParameterString( order.Billing.LastName ),
                DataAccess.CreateParameterString( order.Billing.Company ),
                DataAccess.CreateParameterString( order.Billing.Address1 ),
                DataAccess.CreateParameterString( order.Billing.Address2 ),
                DataAccess.CreateParameterString( order.Billing.City ),
                DataAccess.CreateParameterString( order.Billing.State ),
                DataAccess.CreateParameterString( order.Billing.Zip ),
                DataAccess.CreateParameterString( order.Billing.Country ),
                DataAccess.CreateParameterString( order.Billing.Phone ),
                DataAccess.CreateParameterString( order.Billing.Fax ),
                DataAccess.CreateParameterString( order.Email ),
                DataAccess.CreateParameterString( order.Shipping.FirstName ),
                DataAccess.CreateParameterString( order.Shipping.LastName ),
                DataAccess.CreateParameterString( order.Shipping.Company ),
                DataAccess.CreateParameterString( order.Shipping.Address1 ),
                DataAccess.CreateParameterString( order.Shipping.Address2 ),
                DataAccess.CreateParameterString( order.Shipping.City ),
                DataAccess.CreateParameterString( order.Shipping.State ),
                DataAccess.CreateParameterString( order.Shipping.Zip ),
                DataAccess.CreateParameterString( order.Shipping.Country ),
                DataAccess.CreateParameterString( order.Shipping.Phone ),
                DataAccess.CreateParameterString( order.Shipping.Fax ),
                DataAccess.CreateParameterBool( order.Shipping.Residential ),
                DataAccess.CreateParameterString( order.PaymentMethod ),
                DataAccess.CreateParameterString( order.GatewayPaymentStatus ),
                DataAccess.CreateParameterString( order.GatewayOrderID ),
                DataAccess.CreateParameterString( order.ShippingMethod ),
                DataAccess.CreateParameterBool( order.PaymentComplete ),
                DataAccess.CreateParameterBool( order.Processed ),
                DataAccess.CreateParameterString( order.Status ),
                DataAccess.CreateParameterBool( order.Cancelled ),
                DataAccess.CreateParameterString( order.IPAddress ),
                DataAccess.CreateParameterDecimalMoney( order.Subtotal ),
                DataAccess.CreateParameterDecimalMoney( order.Tax ),
                DataAccess.CreateParameterDecimalMoney( order.ShippingCost ),
                DataAccess.CreateParameterString( order.CouponID ),
                DataAccess.CreateParameterDecimalMoney( order.CouponDiscount ),
                DataAccess.CreateParameterString( order.CustomerComments ),
                DataAccess.CreateParameterString( order.BaseCurrencyCode ),
                DataAccess.CreateParameterString( order.UserCurrencyCode ),
                DataAccess.CreateParameterDouble( order.UserConversionRate ),
                DataAccess.CreateParameterString( order.InvoiceNotes ),
                DataAccess.CreateParameterString( order.GiftCertificateCode ),
                DataAccess.CreateParameterDecimal( order.GiftCertificate ),
                DataAccess.CreateParameterString( order.TrackingNumber ),
                DataAccess.CreateParameterString( order.TrackingMethod ),
                DataAccess.CreateParameterDecimalMoney( order.HandlingFee ),
                DataAccess.CreateParameterBool( order.ContainsRecurring ),
                DataAccess.CreateParameterString( order.AvsAddrStatus ),
                DataAccess.CreateParameterString( order.AvsZipStatus ),
                DataAccess.CreateParameterString( order.CvvStatus ),
                DataAccess.CreateParameterString( order.StoreID.ToString() ),
                DataAccess.CreateParameterString( order.QBTxnID ),
                DataAccess.CreateParameterBool( order.IsQBExported ),
                DataAccess.CreateParameterString( order.PONumber ),
                DataAccess.CreateParameterBool( order.IsTaxExempt ),
                DataAccess.CreateParameterString( order.TaxExemptID ),
                DataAccess.CreateParameterString( order.TaxExemptCountry ),
                DataAccess.CreateParameterString( order.TaxExemptState ),
                DataAccess.CreateParameterInt32( order.RedeemPoint ),
                DataAccess.CreateParameterDecimalMoney( order.RedeemPrice ),
                DataAccess.CreateParameterInt32( order.PointEarned ),
                DataAccess.CreateParameterBool( order.IsPointApplied ),
                DataAccess.CreateParameterBool( order.IsSubscriptionApplied ),
                DataAccess.CreateParameterString( order.OrderID )
                );
            return order;
        }

        private void DeleteOrderAndRelatedDetails( string orderID )
        {
            DataAccessContext.GiftCertificateRepository.DeleteByOrderID( orderID );
            DataAccessContext.OrderItemRepository.DeleteByOrderID( orderID );

            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM OrderNote " +
                "WHERE OrderID = @OrderID; " +

                "DELETE " +
                "FROM OrderTracking " +
                "WHERE OrderID = @OrderID; " +

                "DELETE " +
                "FROM Orders " +
                "WHERE OrderID = @OrderID; ",

                DataAccess.CreateParameterString( orderID ),
                DataAccess.CreateParameterString( orderID ),
                DataAccess.CreateParameterString( orderID ) );
        }

        private string CreateTotalSqlNoAs()
        {
            return " (Subtotal + Tax + ShippingCost + HandlingFee - CouponDiscount - RedeemPrice - GiftCertificate) ";
        }

        private string CreateTotalSql()
        {
            return CreateTotalSqlNoAs() + " AS Total ";
        }

        private string CreateSearchFieldName( string fieldName )
        {
            if (String.Compare( fieldName.Trim(), "Total", true ) == 0)
                return CreateTotalSqlNoAs();
            else
                return fieldName;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.ToLower().Trim().StartsWith( "total" ))
                return "ORDER BY " +
                    sortBy.ToLower().Replace( "total", "(Subtotal + Tax + ShippingCost + HandlingFee - CouponDiscount - RedeemPrice - GiftCertificate)" );
            else
                return "ORDER BY " + sortBy;
        }

        private DateTime ConvertToDateTime( object obj )
        {
            if (obj == DBNull.Value || obj == null || obj.ToString() == "")
                return DateTime.MinValue;
            else
                return Convert.ToDateTime( obj );
        }

        private Order CreateOneFromRow( DataRow row )
        {
            Order orders = new Order();

            orders.OrderID = row["OrderID"].ToString();
            orders.CustomerID = row["CustomerID"].ToString();
            orders.OrderDate = ConvertToDateTime( row["OrderDate"] );
            orders.UserName = row["UserName"].ToString();
            orders.Billing = new Address( row["FirstName"].ToString(), row["LastName"].ToString(),
                row["Company"].ToString(), row["Address1"].ToString(), row["Address2"].ToString(),
                row["City"].ToString(), row["State"].ToString(), row["Zip"].ToString(),
                row["Country"].ToString(), row["Phone"].ToString(), row["Fax"].ToString() );
            orders.Email = row["Email"].ToString();
            orders.Shipping = new ShippingAddress( new Address(
                row["ShippingFirstName"].ToString(),
                row["ShippingLastName"].ToString(),
                row["ShippingCompany"].ToString(),
                row["ShippingAddress1"].ToString(),
                row["ShippingAddress2"].ToString(),
                row["ShippingCity"].ToString(),
                row["ShippingState"].ToString(),
                row["ShippingZip"].ToString(),
                row["ShippingCountry"].ToString(),
                row["ShippingPhone"].ToString(),
                row["ShippingFax"].ToString() ),
                ConvertUtilities.ToBoolean( row["ShippingResidential"] ) );
            orders.CustomerComments = row["CustomerComments"].ToString();
            orders.ShippedDate = ConvertToDateTime( row["ShippedDate"] );
            orders.ShippingMethod = row["ShippingMethod"].ToString();
            orders.PaymentMethod = row["PaymentMethod"].ToString();
            orders.IPAddress = row["IPAddress"].ToString();
            orders.CouponID = row["CouponID"].ToString();
            orders.PaymentMethod = row["PaymentMethod"].ToString();
            orders.GatewayPaymentStatus = row["GatewayPaymentStatus"].ToString();
            orders.GatewayOrderID = row["GatewayOrderID"].ToString();
            orders.PaymentComplete = ConvertUtilities.ToBoolean( row["PaymentComplete"] );
            orders.Processed = ConvertUtilities.ToBoolean( row["Processed"] );
            orders.Cancelled = ConvertUtilities.ToBoolean( row["Cancelled"] );
            orders.Status = row["Status"].ToString();
            orders.Subtotal = ConvertUtilities.ToDecimal( row["Subtotal"] );
            orders.Tax = ConvertUtilities.ToDecimal( row["Tax"] );
            orders.ShippingCost = ConvertUtilities.ToDecimal( row["ShippingCost"] );
            orders.CouponDiscount = ConvertUtilities.ToDecimal( row["CouponDiscount"] );
            orders.BaseCurrencyCode = row["BaseCurrencyCode"].ToString();
            orders.UserCurrencyCode = row["UserCurrencyCode"].ToString();
            orders.UserConversionRate = ConvertUtilities.ToDouble( row["UserConversionRate"] );
            orders.InvoiceNotes = row["InvoiceNotes"].ToString();
            orders.GiftCertificateCode = row["GiftCertificateCode"].ToString();
            orders.GiftCertificate = ConvertUtilities.ToDecimal( row["GiftCertificate"] );
            orders.TrackingNumber = row["TrackingNumber"].ToString();
            orders.TrackingMethod = row["TrackingMethod"].ToString();
            orders.GiftRegistryID = row["GiftRegistryID"].ToString();
            orders.ShowShippingAddress = ConvertUtilities.ToBoolean( row["ShowShippingAddress"] );
            orders.ShippingID = row["ShippingID"].ToString();
            orders.HandlingFee = ConvertUtilities.ToDecimal( row["HandlingFee"] );
            orders.QbPostDate = ConvertUtilities.ToDateTime( row["QbPostDate"] );
            orders.QbReferenceCode = row["QbReferenceCode"].ToString();
            orders.ContainsRecurring = ConvertUtilities.ToBoolean( row["ContainsRecurring"] );
            orders.ParentID = row["ParentID"].ToString();
            orders.PaymentToken = row["PaymentToken"].ToString();
            orders.AvsAddrStatus = row["AvsAddrStatus"].ToString();
            orders.AvsZipStatus = row["AvsZipStatus"].ToString();
            orders.CvvStatus = row["CvvStatus"].ToString();
            orders.StoreID = row["StoreID"].ToString();
            orders.QBTxnID = row["QBTxnID"].ToString();
            orders.IsQBExported = ConvertUtilities.ToBoolean( row["IsQBExported"] );
            orders.PONumber = ConvertUtilities.ToString( row["PONumber"] );
            orders.IsTaxExempt = ConvertUtilities.ToBoolean( row["IsTaxExempt"] );
            orders.TaxExemptID = row["TaxExemptID"].ToString();
            orders.TaxExemptCountry = row["TaxExemptCountry"].ToString();
            orders.TaxExemptState = row["TaxExemptState"].ToString();
            orders.RedeemPoint = ConvertUtilities.ToInt32( row["RedeemPoint"] );
            orders.RedeemPrice = ConvertUtilities.ToDecimal( row["RedeemPrice"] );
            orders.PointEarned = ConvertUtilities.ToInt32( row["PointEarned"] );
            orders.IsPointApplied = ConvertUtilities.ToBoolean( row["IsPointApplied"] );
            orders.IsSubscriptionApplied = ConvertUtilities.ToBoolean( row["IsSubscriptionApplied"] );

            return orders;
        }

        private string CreateStoreFilter( string storeID )
        {
            if (!String.IsNullOrEmpty( storeID ))
                return "AND StoreID = " + storeID + " ";
            else
                return " ";
        }

        private DataTable SearchOrderPrivate(
            string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            params DbParameter[] parameterExtraFilter
            )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, CreateSearchFieldName( searchFilter.FieldName ), extraFilter, parameterList );

            foreach (DbParameter paramExtra in parameterExtraFilter)
            {
                parameterList.Add( paramExtra );
            }

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT *, " + CreateTotalSql() +
                "FROM Orders " +
                commandSearch +
                CreateStoreFilter( storeID ) +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return full;
        }

        private DataTable SearchOrderPrivate(
            string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            int startIndex,
            int endIndex,
            params DbParameter[] parameterExtraFilter
            )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, CreateSearchFieldName( searchFilter.FieldName ), extraFilter, parameterList );

            foreach (DbParameter paramExtra in parameterExtraFilter)
            {
                parameterList.Add( paramExtra );
            }

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM (" +
                "SELECT *, " + CreateTotalSql() +
                "FROM Orders " +
                commandSearch +
                CreateStoreFilter( storeID ) +
                //CreateSortByText(sortBy) + 
                ") AS SearchOrder ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return table;
        }

        private int GetCountSearchOrderPrivate(
            string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            params DbParameter[] parameterExtraFilter
            )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, CreateSearchFieldName( searchFilter.FieldName ), extraFilter, parameterList );

            foreach (DbParameter paramExtra in parameterExtraFilter)
            {
                parameterList.Add( DataAccess.CreateParameterString( paramExtra.Value.ToString() ) );
            }

            string countResult = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) AS Row " +
                "FROM Orders " +
                commandSearch +
                CreateStoreFilter( storeID ),
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return Convert.ToInt32( countResult );
        }

        private DataTable SearchOrderPrivate(
            string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter )
        {
            DataTable full = SearchOrderPrivate( sortBy, searchFilter, extraFilter, storeID, parameterExtraFilter );
            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["OrderID"] };
            full.PrimaryKey = keys;
            return full;
        }

        private string CreateProcessedFilter( BoolFilter processStatus )
        {
            if (processStatus == BoolFilter.ShowTrue)
            {
                return "Processed = " + DataAccess.CreateLiteralBool( true );
            }
            else if (processStatus == BoolFilter.ShowFalse)
            {
                return "Processed = " + DataAccess.CreateLiteralBool( false );
            }
            else
            {
                return "1 = 1";
            }
        }

        private string CreatePaymentFilter( BoolFilter paymentStatus )
        {
            if (paymentStatus == BoolFilter.ShowTrue)
            {
                return "PaymentComplete = " + DataAccess.CreateLiteralBool( true );
            }
            else if (paymentStatus == BoolFilter.ShowFalse)
            {
                return "PaymentComplete = " + DataAccess.CreateLiteralBool( false );
            }
            else
            {
                return "1 = 1 ";
            }
        }

        private string CreateExtraFilter( OrderCreateExtraFilter extraFilter )
        {


            string result = String.Empty;

            result += CreateProcessedFilter( extraFilter.ProcessStatus );
            result += " AND ";
            result += CreatePaymentFilter( extraFilter.PaymentStatus );

            return result;
        }

        private IList<Order> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<Order> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Order> list = new List<Order>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private IList<Order> ConvertTableToList( DataTable table, int startIndex, int endIndex, int row )
        {
            IList<Order> list = new List<Order>();

            for (int i = 0; i < row; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private void InsertDummyData()
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO Orders (UserName) VALUES ('a')" );
        }

        private void DeleteDummyData( int orderID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM Orders WHERE OrderID = @OrderID;",
                DataAccess.CreateParameterInt32( orderID ) );
        }

        private void SqlSetIdentity( int orderID )
        {
            int value = orderID - 1;
            DataAccess.ExecuteNonQuery(
                "DBCC CHECKIDENT (Orders, RESEED, " + value.ToString() + ")" );
        }

        private string CreateFromDateWithAnd( DateTime? from, ArrayList parameters )
        {
            if (from == null)
                return String.Empty;

            DateTime from12am = new DateTime( from.Value.Year, from.Value.Month, from.Value.Day, 0, 0, 0 );
            parameters.Add( DataAccess.CreateParameterDateTime( from12am ) );
            return " AND OrderDate >= @FromOrderDate ";
        }

        private string CreateToDateWithAnd( DateTime? to, ArrayList parameters )
        {
            if (to == null)
                return String.Empty;

            DateTime toLastSecond = new DateTime( to.Value.Year, to.Value.Month, to.Value.Day, 23, 59, 59 );
            parameters.Add( DataAccess.CreateParameterDateTime( toLastSecond ) );
            return " AND OrderDate <= @ToOrderDate ";
        }

        private static string GenerateCondition( int startOrderID, int numberOfDays, ArrayList parameterLists )
        {
            string condition = ";";
            if (startOrderID > 0)
            {
                condition = "WHERE OrderID > @StartOrderID;";
                parameterLists.Add( DataAccess.CreateParameterInt32( startOrderID ) );
            }
            else if (numberOfDays > 0)
            {
                condition = "WHERE " + DbImplementation.Sql.DateDiffWithToday( "OrderDate" ) + " <= @NumberOfDays ;";
                parameterLists.Add( DataAccess.CreateParameterInt32( numberOfDays ) );
            }
            return condition;
        }

        #endregion

        #region Public Methods

        public Order Save( Order orders )
        {
            Order result;

            if (orders.OrderID == "0")
                result = Create( orders );
            else
                result = Update( orders );

            return result;
        }

        public Order GetOne( string orderID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT *, " + CreateTotalSql() +
                "FROM Orders " +
                "WHERE OrderID = @OrderID; ",
                DataAccess.CreateParameterString( orderID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return null;
        }

        public void Delete( string orderID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DeleteOrderAndRelatedDetails( orderID );
                transaction.Complete();
            }
        }

        public IList<Order> SearchOrder(
            string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            int startIndex,
            int endIndex,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter )
        {
            //DataTable full = SearchOrderPrivate(sortBy, searchFilter, extraFilter, storeID, out howManyItems, parameterExtraFilter);
            //return ConvertTableToList(full, startIndex, endIndex);
            DataTable full = SearchOrderPrivate(
                sortBy,
                searchFilter,
                extraFilter,
                storeID,
                startIndex,
                endIndex,
                parameterExtraFilter );
            howManyItems = GetCountSearchOrderPrivate(
                sortBy,
                searchFilter,
                extraFilter,
                storeID,
                parameterExtraFilter );
            return ConvertTableToList( full, startIndex, endIndex, full.Rows.Count );
        }

        public IList<Order> SearchOrder(
            string sortBy,
            SearchFilter searchFilter,
            OrderCreateExtraFilter extraFilter,
            string storeID,
            int startIndex,
            int endIndex,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter )
        {
            DataTable full = SearchOrderPrivate(
                sortBy,
                searchFilter,
                CreateExtraFilter( extraFilter ),
                storeID,
                startIndex,
                endIndex,
                parameterExtraFilter );
            howManyItems = GetCountSearchOrderPrivate(
                sortBy,
                searchFilter,
                CreateExtraFilter( extraFilter ),
                storeID,
                parameterExtraFilter );
            return ConvertTableToList( full, startIndex, endIndex, full.Rows.Count );
        }

        public IList<Order> SearchOrder(
            string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter )
        {
            DataTable full = SearchOrderPrivate(
                sortBy, searchFilter, extraFilter, storeID, out howManyItems, parameterExtraFilter );
            return ConvertTableToList( full );
        }

        public IList<Order> SearchOrder(
            string sortBy,
            SearchFilter searchFilter,
            OrderCreateExtraFilter extraFilter,
            string storeID,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter )
        {
            DataTable full = SearchOrderPrivate(
                sortBy,
                searchFilter,
                CreateExtraFilter( extraFilter ),
                storeID,
                out howManyItems,
                parameterExtraFilter );
            return ConvertTableToList( full );
        }

        public IList<Order> ExportOrder(
            string sortBy,
            SearchFilter searchFilter,
            OrderCreateExtraFilter extraFilter,
            string storeID,
            params DbParameter[] parameterExtraFilter
            )
        {
            return ConvertTableToList(
                SearchOrderPrivate( sortBy, searchFilter, CreateExtraFilter( extraFilter ), storeID, parameterExtraFilter ) );
        }

        public decimal SumOrder(
            SearchFilter searchFilter,
            OrderCreateExtraFilter extrafilter,
            string storeID )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, CreateSearchFieldName( searchFilter.FieldName ), CreateExtraFilter( extrafilter ), parameterList );

            string totalPrice = DataAccess.ExecuteScalar(
                "SELECT SUM(Subtotal + Tax + ShippingCost + HandlingFee - CouponDiscount - GiftCertificate) " +
                "FROM Orders " +
                commandSearch +
                CreateStoreFilter( storeID ) +
                ";",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            return ConvertUtilities.ToDecimal( totalPrice );
        }

        public IList<Order> GetOrdersByProcessedAndStatus( bool processed, string status )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT *, " + CreateTotalSql() + " FROM Orders WHERE Processed = @Processed AND Status = @Status;",
                DataAccess.CreateParameterBool( processed ),
                DataAccess.CreateParameterString( status ) );
            return ConvertTableToList( table );
        }

        public IList<Order> GetPaidOrdersByDateTime( DateTime? from, DateTime? to )
        {
            ArrayList parameters = new ArrayList();

            parameters.Add( DataAccess.CreateParameterBool( true ) );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT *, " + CreateTotalSql() + " " +
                "FROM Orders " +
                "WHERE PaymentComplete = @PaymentComplete " +
                CreateFromDateWithAnd( from, parameters ) +
                CreateToDateWithAnd( to, parameters ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameters ) );
            return ConvertTableToList( table );
        }

        public IList<Order> GetOrder(
            int startOrderID,
            int numberOfDays,
            int limitCount )
        {
            ArrayList parameterList = new ArrayList();
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT *, " + CreateTotalSql() +
                "FROM Orders " +
                GenerateCondition( startOrderID, numberOfDays, parameterList ),
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            DataColumn[] keys = new DataColumn[] { table.Columns["OrderID"] };
            table.PrimaryKey = keys;
            return ConvertTableToList( table, 0, limitCount );
        }

        public IList<Order> GetLastestOrder(int limitCount )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT TOP " + limitCount.ToString() + " *, " + CreateTotalSql() +
                "FROM Orders ORDER BY OrderDate DESC"
                );

            DataColumn[] keys = new DataColumn[] { table.Columns["OrderID"] };
            table.PrimaryKey = keys;
            return ConvertTableToList( table);
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT " + _allFieldNotEncryption + ", " + CreateTotalSql() +
                 "FROM Orders " +
                 "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<TableSchemaItem> GetTableOrderHistorySchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OrderID, OrderDate, TrackingNumber, " + CreateTotalSql() +
                "FROM Orders " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public string GetMaximumOrderID()
        {
            string orderID = DataAccess.ExecuteScalar(
                "SELECT max(orderid) FROM Orders" );

            if (!String.IsNullOrEmpty( orderID ))
                return orderID;
            else
                return "0";
        }

        public void SetStartupOrderID( string startOrderID )
        {
            int maxOrderID = ConvertUtilities.ToInt32( GetMaximumOrderID().ToString() );
            int startNumber = ConvertUtilities.ToInt32( startOrderID );

            if (startNumber > maxOrderID)
            {
                switch (DataAccessHelper.DbProviderName.ToLower())
                {
                    case "system.data.oledb":
                        using (DataTransactionScope scope = new DataTransactionScope())
                        {
                            do
                            {
                                InsertDummyData();
                                maxOrderID++;
                                DeleteDummyData( maxOrderID );
                            }
                            while (maxOrderID != startNumber - 1);

                            scope.Complete();
                        }
                        break;

                    case "system.data.sqlclient":
                        SqlSetIdentity( startNumber );
                        break;
                }
            }
        }

        public string GetOrderIDByGatewayID( string gatewayOrderID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT *, " + CreateTotalSql() +
                "FROM Orders " +
                "WHERE GatewayOrderID = @GatewayOrderID;",
                DataAccess.CreateParameterString( gatewayOrderID ) );

            if (table.Rows.Count == 0)
                return "0";
            else
                return table.Rows[0]["OrderID"].ToString();

        }

        public IList<Order> GetByIsQBExported( bool qbIsExported, string storeID, string existList )
        {
            string query = "SELECT * FROM Orders " +
                "WHERE IsQBExported = @IsQBExported AND StoreID = @storeID";

            if (!string.IsNullOrEmpty( existList ))
            {
                query += " AND QBTxnID NOT IN ( " + existList + "  ); ";
            }
            else
            {
                query += ";";
            }

            DataTable table = DataAccess.ExecuteSelect( query,
                DataAccess.CreateParameterBool( qbIsExported ),
                DataAccess.CreateParameterString( storeID )
            );

            return ConvertTableToList( table );
        }

        public void UpdateQBData( string orderID, string qbTxnID, bool isQBExported )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Orders " +
                "SET QBTxnID = @QBTxnID, " +
                "IsQBExported = @IsQBExported " +
                "WHERE OrderID = @OrderID;",
                DataAccess.CreateParameterString( qbTxnID ),
                DataAccess.CreateParameterBool( isQBExported ),
                DataAccess.CreateParameterString( orderID )
            );
        }

        public void SetDefaultQBData( string qbTxnID )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Order " +
                "SET QBTxnID = '', " +
                "IsQBExported = @IsQBExported " +
                "WHERE QBTxnID = @QBTxnID;",
                DataAccess.CreateParameterBool( false ),
                DataAccess.CreateParameterString( qbTxnID )
            );
        }

        #endregion
    }
}
