using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Payments;
using Vevo.Shared.DataAccess;
using System.Data;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class PayPalIpnRepositoryNoCache : IPayPalIpnRepository
    {
        #region Private

        private const string AllPayPalIpnFields = " PayPalIpnID, ReferenceID, IsRecurringProfileCreated, IpnResponse, ReceiveDate ";

        private PayPalIpn Create( PayPalIpn payPalIpn )
        {
            string newPayPalIpnID = DataAccess.ExecuteScalar(
                "INSERT INTO PayPalIpn ( ReferenceID, IsRecurringProfileCreated, IpnResponse, ReceiveDate ) " +
                "VALUES ( @ReferenceID, @IsRecurringProfileCreated, @IpnResponse, @ReceiveDate ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( payPalIpn.ReferenceID ),
                DataAccess.CreateParameterBool( payPalIpn.IsRecurringProfileCreated ),
                DataAccess.CreateParameterString( payPalIpn.IpnResponse ),
                DataAccess.CreateParameterDateTime( payPalIpn.ReceiveDate ) );

            payPalIpn.PayPalIpnID = newPayPalIpnID;
            return payPalIpn;
        }

        private PayPalIpn Update( PayPalIpn payPalIpn )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE PayPalIpn " +
                "SET ReferenceID = @ReferenceID, IsRecurringProfileCreated = @IsRecurringProfileCreated, " +
                "    IpnResponse = @IpnResponse, ReceiveDate = @ReceiveDate " +
                "WHERE PayPalIpnID = @PayPalIpnID; ",
                DataAccess.CreateParameterString( payPalIpn.ReferenceID ),
                DataAccess.CreateParameterBool( payPalIpn.IsRecurringProfileCreated ),
                DataAccess.CreateParameterString( payPalIpn.IpnResponse ),
                DataAccess.CreateParameterDateTime( payPalIpn.ReceiveDate ),
                DataAccess.CreateParameterString( payPalIpn.PayPalIpnID ) );

            return payPalIpn;
        }

        private PayPalIpn CreateOneFromRow( DataRow row )
        {
            PayPalIpn payPalIpn = new PayPalIpn();
            payPalIpn.PayPalIpnID = row["PayPalIpnID"].ToString();
            payPalIpn.ReferenceID = row["ReferenceID"].ToString();
            payPalIpn.IpnResponse = row["IpnResponse"].ToString();
            payPalIpn.IsRecurringProfileCreated = ConvertUtilities.ToBoolean( row["IsRecurringProfileCreated"] );
            payPalIpn.ReceiveDate = ConvertUtilities.ToDateTime( row["ReceiveDate"] );

            return payPalIpn;
        }

        #endregion

        #region Public

        public PayPalIpn Save( PayPalIpn payPalIpn )
        {
            PayPalIpn result;

            if (payPalIpn.PayPalIpnID == "0")
                result = Create( payPalIpn );
            else
                result = Update( payPalIpn );

            return result;
        }

        public PayPalIpn GetOne( string payPalIpnID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllPayPalIpnFields +
                "FROM PayPalIpn " +
                "WHERE PayPalIpnID = @PayPalIpnID; ",
                DataAccess.CreateParameterString( payPalIpnID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return PayPalIpn.Null;
        }

        public string GetIpnIDByReferenceID( string referenceID )
        {
            return DataAccess.ExecuteScalar(
                "SELECT PayPalIpnID " +
                "FROM PayPalIpn " +
                "WHERE ReferenceID = @ReferenceID;",
                DataAccess.CreateParameterString( referenceID ) );
        }

        public void Delete( string payPalIpnID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM PayPalIpn " +
                "WHERE PayPalIpnID = @PayPalIpnID; ",
                DataAccess.CreateParameterString( payPalIpnID ) );
        }

        #endregion
    }
}
