using com.paypal.sdk.services;
using com.paypal.sdk.util;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Globalization;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Security;

using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.Domain.Tax;
using Vevo.Domain.Users;
using Vevo.WebAppLib;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;
using Vevo.Deluxe.Domain;
using Vevo.Deluxe.Domain.CustomerReward;

namespace Vevo.Domain.Payments.PayPalProUS
{
    /// <summary>
    /// Summary description for PayPalProPayment
    /// </summary>
    ///

    public class PayPalProUSPaymentMethod : OnPayPalProUSPaymentMethod
    {
        #region Private

        private static string CreateErrorMessage( NVPCodec decoder )
        {
            return "<b>ErrorCode</b><br /> " + decoder["L_ERRORCODE0"] + "<br />" +
                "<b>Description</b><br />" + decoder["L_SHORTMESSAGE0"] + " - " + decoder["L_LONGMESSAGE0"];
        }

        private static string CreateErrorMessage( string productName, NVPCodec decoder )
        {
            return "Productname : " + productName + "<br/>" + CreateErrorMessage( decoder );
        }

        private static string CreateInformationMerchant( string emailCustomer, string username )
        {
            string result = String.Empty;

            result = "*******************************************\r\n";
            result += "Username: " + username + "\r\n";
            result += "Email Customer: " + emailCustomer + "\r\n";
            result += "*******************************************\r\n";
            return result;
        }

        private static void SendEmailCancelRecurring( string emailCustomer, string username, string orderID, string companyEmail )
        {
            StreamReader vevoReadText;
            string bodyMail;
            using (vevoReadText =
                File.OpenText(
                    HttpContext.Current.Server.MapPath( "~/ContentTemplates/" )
                    + "RecurringCanceledConfirmation.txt" ))
            {
                bodyMail = vevoReadText.ReadToEnd();
            }

            IList<OrderItem> orderItemList = DataAccessContext.OrderItemRepository.GetByOrderID( orderID );
            string productName = String.Empty;

            if (orderItemList.Count > 0)
                productName = orderItemList[0].Name.Replace( "<br>", "\r\n" );

            bodyMail = bodyMail.Replace( "[OrderID]", orderID );
            bodyMail = bodyMail.Replace( "[ProductName]", productName );
            bodyMail = bodyMail.Replace( "[CancelDate]", DateTime.Today.ToShortDateString() );

            WebUtilities.SendMail(
                companyEmail,
                emailCustomer,
                "Recurring Canceled Confirmation",
                bodyMail );

            bodyMail = CreateInformationMerchant( emailCustomer, username ) + bodyMail;

            WebUtilities.SendMail(
                companyEmail,
                companyEmail,
                "Recurring Canceled Confirmation",
                bodyMail );
        }

        private static OrderItem GetParentOrderItem( string recurringID )
        {
            string orderItemID = DataAccessContext.RecurringProfileRepository.GetOrderItemIDByReferenceID( recurringID );
            return DataAccessContext.OrderItemRepository.GetOne( orderItemID );
        }

        private static void UpdateRecurringProfile( RecurringProfile recurringProfile, DateTime updateTime )
        {
            recurringProfile.RecurringSequence += 1;
            recurringProfile.UpdateTime = updateTime;
            DataAccessContext.RecurringProfileRepository.Save( recurringProfile );
        }

        private static Order CreateChildOrder(
            RecurringProfile recurringProfile,
            decimal itemPrice,
            Order parentOrder,
            string gatewayPaymentStatus,
            string recurringPeriod )
        {
            decimal recurringTax = 0m;
            decimal recurringShipping = 0m;
            decimal recurringDiscount = 0m;


            if (recurringPeriod == "Regular")
            {
                recurringTax = recurringProfile.RegularTax;
                recurringShipping = recurringProfile.RegularShipping;
                recurringDiscount = recurringProfile.RegularDiscount;
            }
            else if (recurringPeriod == "Trial")
            {
                recurringTax = recurringProfile.TrialTax;
                recurringShipping = recurringProfile.TrialShipping;
                recurringDiscount = recurringProfile.TrialDiscount;
            }

            itemPrice = itemPrice + recurringDiscount - recurringShipping - recurringTax;

            Order order = (Order) parentOrder.Clone();

            string status = "New";

            order.OrderID = "0";
            order.Status = status;
            order.Subtotal = itemPrice;
            order.Tax = recurringTax;
            order.ShippingCost = recurringShipping;
            order.CouponID = "";
            order.CouponDiscount = recurringDiscount;
            order.GiftCertificateCode = "";
            order.GiftCertificate = 0m;
            order.GiftRegistryID = "";
            order.HandlingFee = 0m;
            order.ParentID = parentOrder.OrderID;

            order.GatewayOrderID = order.OrderID;      // May not need to set (???)
            order.GatewayPaymentStatus = gatewayPaymentStatus;

            if (gatewayPaymentStatus.ToLower() == "completed")
            {
                order.PaymentComplete = true;
                CustomerRewardPoint.UpdateRewardPoint( order );
            }
            else
                order.PaymentComplete = false;

            return DataAccessContext.OrderRepository.Save( order );
        }

        private static void CreateChildOrderItem(
            RecurringProfile recurringProfile,
            Order newOrder,
            OrderItem parentOrderItem,
            string recurringStatus,
            string recurringPeriod,
            string recurringReceiptID,
            DateTime updateTime )
        {
            OrderItem orderItem = (OrderItem) parentOrderItem.Clone();

            OrderItemRecurring orderItemRecurring = new OrderItemRecurring();

            orderItem.OrderItemID = "0";
            orderItem.OrderID = newOrder.OrderID;
            orderItem.RecurringID = recurringProfile.RecurringID;
            orderItem = DataAccessContext.OrderItemRepository.Save( orderItem );
            string newOrderItemID = orderItem.OrderItemID;

            orderItemRecurring.OrderItemID = newOrderItemID;
            orderItemRecurring.RecurringStatus = recurringStatus;
            orderItemRecurring.RecurringPeriod = recurringPeriod;
            orderItemRecurring.UpdateTime = updateTime;
            orderItemRecurring.RecurringReceiptID = recurringReceiptID;
            DataAccessContext.OrderItemRepository.UpdateRecurringItem( orderItemRecurring );
        }


        #endregion

        #region Public Method

        public PayPalProUSPaymentMethod()
            : base()
        {
        }

        public override string Name
        {
            get
            {
                return "PayPal Pro";
            }
        }

        public enum RecurringStatus
        {
            Cancel,
            Suspend,
            Reactivate
        }

        public override bool ProcessPayment(
            decimal amount,
            Currency currency,
            Culture culture,
            CheckoutDetails checkout,
            out ProcessPaymentResult result )
        {
            throw new Exception();
        }

        public static bool ManageRecurring(
            string profileID,
            string emailCustomer,
            string username,
            string orderID,
            RecurringStatus action,
            string companyEmail,
            string note,
            out string errorMessage )
        {
            errorMessage = String.Empty;
            NVPCallerServices caller = PayPalProUSUtilities.CreatePayPalAPI();
            NVPCodec encoder = new NVPCodec();
            encoder["METHOD"] = "ManageRecurringPaymentsProfileStatus";
            encoder["ACTION"] = action.ToString();
            encoder["NOTE"] = note;
            encoder["PROFILEID"] = profileID;

            string pStrrequestforNvp = encoder.Encode();
            string pStresponsenvp = caller.Call( pStrrequestforNvp );

            NVPCodec decoder = new NVPCodec();
            decoder.Decode( pStresponsenvp );

            string strAck = decoder["ACK"];
            if (strAck != null && (strAck == "Success" || strAck == "SuccessWithWarning"))
            {
                SendEmailCancelRecurring( emailCustomer, username, orderID, companyEmail );
                return true;
            }
            else
            {
                errorMessage = CreateErrorMessage( decoder );
                return false;
            }
        }

        public static string CreateChildOrderByPayPalRefenceID(
            string referenceID,
            decimal itemPrice,
            string recurringStatus,
            string gatewayPaymentstatus,
            string recurringReceiptID,
            string recurringPeriod,
            DateTime updateTime,
            out int allSequence )
        {
            allSequence = 0;

            OrderItem parentOrderItem = GetParentOrderItem( referenceID );
            if (parentOrderItem == null)
                return "0";

            if (!(ConvertUtilities.ToInt32( parentOrderItem.OrderID ) > 0))
                return "0";

            RecurringProfile recurringProfile = DataAccessContext.RecurringProfileRepository.GetOne( parentOrderItem.RecurringID );

            Order order = DataAccessContext.OrderRepository.GetOne( parentOrderItem.OrderID );
            if (recurringProfile.RecurringSequence > 0)
            {
                order = CreateChildOrder(
                    recurringProfile, itemPrice, order, gatewayPaymentstatus, recurringPeriod );

                string newOrderID = order.OrderID;
                if (ConvertUtilities.ToInt32( newOrderID ) > 0)
                {
                    CreateChildOrderItem(
                        recurringProfile, order, parentOrderItem, recurringStatus, recurringPeriod, recurringReceiptID, updateTime );
                }
            }

            UpdateRecurringProfile( recurringProfile, updateTime );
            allSequence = recurringProfile.RecurringSequence;

            return order.OrderID;
        }

        #endregion
    }

}
