﻿using System;
using System.Collections.Generic;

using Vevo.Domain.Base;
using Vevo.Domain.Products;
using Vevo.Base.Domain;

namespace Vevo.Domain.DataInterfaces
{
    public interface ICategoryRepository
    {
        Category Save( Category category );
        Category SaveWithIdentity( Category category );
        void Delete( string categoryID );
        void DeleteLocalesByCultureID( string cultureID );
        void ReArrangeSortOrder( Culture culture, string parentCategoryID, string categoryID, BoolFilter isEnabled );
        void ReArrangeSortOrderByParentCategoryID( Culture culture, string parentCategoryID, BoolFilter isEnabled );
        void UpdateSortOrder( string[] sortedCategoryIDs );

        Category GetOne( Culture culture, string categoryID );
        Category GetOneByUrlName( Culture culture, string urlName );
        IList<Category> GetAll( Culture culture, string sortBy );
        IList<Category> GetAll( Culture culture, string sortBy, BoolFilter isEnabled );
        IList<Category> GetAll(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Category> GetAllWithoutRootCategory( Culture culture, string sortBy );
        IList<Category> GetAllLeafCategory( Culture culture, string sortBy );
        int GetCountAllWithoutRootCategory();

        IList<Category> GetByParentID(
            Culture culture,
            string parentCategoryID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Category> GetByParentID(
            Culture culture,
            string parentCategoryID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Category> GetByParentIDAndRootID(
            Culture culture,
            string parentCategoryID,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Category> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Category> GetByRootIDLeafOnly(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Category> GetByRootIDNotLeaf(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Category> GetByRootIDNoProduct(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );


        IList<Category> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Category> GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled );

        IList<Category> GetRootCategory(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Category> GetByParentUrlName(
             Culture culture,
             string parentCategoryName,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems );

        IList<ILocale> GetLocales( string categoryID );
        IList<TableSchemaItem> GetTableSchema();
        IList<string> GetLeafFromCategoryID( string parentCategoryID, IList<string> list );

        bool IsLeaf( string categoryID );
        bool IsCategoryIDNotLeaf( string parentCategoryID );
        bool IsUrlNameNotLeaf( string parentCategoryUrlName );
        void RefreshUrlName( string categoryID, string name );
        bool HasCategoryInDiscountGroup( string discountgroupID );

        IList<Category> SearchCategory(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            );

        IList<Category> SearchCategoryByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
            );

        IList<Category> SearchRootCategory(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            );
    }
}
