﻿using System;
using System.Collections.Generic;

using Vevo.Domain.Base;
using Vevo.Domain.Products;
using Vevo.Base.Domain;

namespace Vevo.Domain.DataInterfaces
{
    public interface IDepartmentRepository
    {
        Department Save( Department department );
        Department SaveWithIdentity( Department department );
        void Delete( string departmentID );
        void DeleteLocalesByCultureID( string cultureID );
        void ReArrangeSortOrder( Culture culture, string parentDepartmentID, string departmentID, BoolFilter isEnabled );
        void ReArrangeSortOrderByParentDepartmentID( Culture culture, string parentDepartmentID, BoolFilter isEnabled );
        void UpdateSortOrder( string[] sortedDepartmentIDs );

        Department GetOne( Culture culture, string departmentID );
        Department GetOneByUrlName( Culture culture, string urlName );
        IList<Department> GetAll( Culture culture, string sortBy );
        IList<Department> GetAll( Culture culture, string sortBy, BoolFilter isEnabled );
        IList<Department> GetAll(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Department> GetAllWithoutRootDepartment( Culture culture, string sortBy );
        IList<Department> GetAllLeafDepartment( Culture culture, string sortBy );
        int GetCountAllWithoutRootDepartment();

        IList<Department> GetByParentID(
            Culture culture,
            string parentDepartmentID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Department> GetByParentID(
            Culture culture,
            string parentDepartmentID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Department> GetByParentIDAndRootID(
            Culture culture,
            string parentDepartmentID,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Department> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Department> GetByRootIDLeafOnly(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Department> GetByRootIDNotLeaf(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Department> GetByRootIDNoProduct(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled );

        IList<Department> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Department> GetRootDepartment(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled );

        IList<Department> GetRootDepartment(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems );

        IList<Department> GetByParentUrlName(
             Culture culture,
             string parentDepartmentName,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems );

        IList<ILocale> GetLocales( string departmentID );
        IList<TableSchemaItem> GetTableSchema();
        IList<string> GetLeafFromDepartmentID( string parentDepartmentID, IList<string> list );

        bool IsLeaf( string departmentID );
        bool IsDepartmentIDNotLeaf( string parentDepartmentID );
        bool IsUrlNameNotLeaf( string parentDepartmentUrlName );
        void RefreshUrlName( string departmentID, string name );
        bool HasDepartmentInDiscountGroup( string discountgroupID );

        IList<Department> SearchDepartment(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            );

        IList<Department> SearchDepartmentByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
            );

        IList<Department> SearchRootDepartment(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            );
    }
}
