using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Domain.Base;
using Vevo.Domain.Products;
using System;
using Vevo.Base.Domain;

namespace Vevo.Domain.DataInterfaces
{
    public interface IProductRepository
    {
        // Functions that are intended to be used internally inside aggregates.
        #region Aggregate Methods

        IEntityList<ProductImage> GetProductImages( string productID );
        IList<ProductOptionGroup> GetProductOptionGroups( Culture culture, string productID );
        IList<ProductStock> GetProductStocks( string productID );
        IList<ProductShippingCost> GetProductShippingCosts( string productID );
        IList<ProductPrice> GetProductPrices( string productID );
        IList<ProductPrice> GetDefaultProductPriceList();
        IList<ProductSpecification> GetProductSpecifications( string productID );
        IList<string> GetProductSpecificationsIDByProductIDs( IList<String> productIDs );
        void DeleteProductSpecification();
        ProductPrice GetDefaultProductPrice( string productID );
        void DeleteProductPricesByStoreID( string storeID );
        void CreateProductPricesFromDefaultByStoreID( string storeID );
        IList<ILocale> GetLocales( string productID );
        IList<ILocale> GetImageLocales( string productImageID );
        IList<ILocale> GetMetaInformationsByStoreID( string productID, string storeID );
        IList<ILocale> GetMetaInformations( string productID );
        IList<ProductKit> GetProductKits( Culture culture, string productID );
        IList<ProductSpecification> GetAllProductSpecifications();

        #endregion


        #region Public Methods

        Product Save( Product product );
        void Delete( string productID );
        void DeleteLocalesByCultureID( string cultureID );
        void DeleteAll();
        void DeleteProductMetaInformationByStoreID( string storeID );
        void RefreshProductUrlNameAll( string productName );
        void RefreshProductUrlNameAll();
        Product GetOne( Culture culture, string productID, string storeID );
        string GetProductIDFromUrlName( string urlName );
        bool ImageIsUsedByAnotherProduct( string urlLargeImage );
        bool IsRegularImageExisting( string regularImagePath );
        bool IsLargeImageExisting( string largeImagePath );
        bool IsThumbnailImageExisting( string thumbnailImagePath );
        bool IsImageSecondaryExisting( string imageSecondary );

        string GetProductIDBySku( string sku );
        Dictionary<string, string> GetProductIDAndSku();

        int GetAllProductCount();
        int GetAllProductCountBySku( string sku );
        int GetAllProductInactiveCount();
        int GetAllProductInLowStockCount( int lowStock, bool useStockInventoryControl );

        int GetProductCountByCategoryID( string categoryID, BoolFilter isEnabled );
        int GetProductCountByCategoryID( string[] categoryIDs );
        int GetProductCountByDepartmentID( string departmentID, BoolFilter isEnabled );
        int GetProductCountByDepartmentID( string[] departmentIDs );
        int GetProductCountByManufacturerID( string manufacturerID, BoolFilter isEnabled );
        Dictionary<string, int> GetProductCountGroupByCategoryID();
        Dictionary<string, int> GetProductCountGroupByDepartmentID();
        int GetFacetCount(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            );
        decimal GetMaxPriceByCategoryDepartment(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            BoolFilter isEnabled,
            string storeID
            );

        IList<String> GetProductIDListByCategoryDepartment(
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            BoolFilter isEnabled,
            string storeID );

        IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
            );

        IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            BoolFilter isEnabled,
            bool isAffiliate,
            string storeID
            );

        IList<Product> GetByDepartmentID(
            Culture culture,
            string departmentID,
            string sortBy,
            BoolFilter isEnabled,
            string storeID
        );

        IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            bool isIncludeProductKit,
            out int howmanyItems,
            string storeID
            );

        IList<Product> GetByCategoryID(
            Culture culture,
            string categoryID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
            );

        IList<Product> GetByManufacturerID(
            Culture culture,
            string manufacturerID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
            );

        IList<Product> GetFacetResultByCategoryID(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            string manufacturerID,
            string minPrice,
            string maxPrice,
            IList<SpecificationItemValue> specValueList,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
            );

        IList<Product> GetNewArrivalByCategoryDepartmentID(
            Culture culture,
            string[] categoryIDs,
            string[] departmentIDs,
            int limitCount,
            string storeID );

        IList<Product> GetNewArrivalByManufacturerID(
            Culture culture,
            string manufacturerID,
            int limitCount,
            string storeID );

        IList<Product> GetByDepartmentID(
            Culture culture,
            string departmentID,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
        );

        IList<Product> GetByDepartmentID(
            Culture culture,
            string[] departmentIDs,
            string sortBy,
            int startIndex,
            int endIndex,
            BoolFilter isEnabled,
            out int howmanyItems,
            string storeID
        );

        void RefreshIsParentVisibleByCategoryID( string categoryID, string storeID );
        void RefreshIsParentVisibleByDepartmentID( string departmentID, string storeID );

        IList<Product> GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
            );

        IList<Product> GetAllByBestSelling(
            Culture culture,
            int numberOfResult,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
            );

        IList<Product> GetAllByRandom(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
            );

        IList<Product> GetAllByRandom(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        );

        IList<Product> GetAllByNewArrival(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
        );

        IList<Product> GetAllByNewArrival(
            Culture culture,
            int numberOfRandomProducts,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        );

        IList<Product> GetAllByTodaySpecial(
            Culture culture,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit
            );

        IList<Product> GetAllByTodaySpecial(
            Culture culture,
            int outOfStockValue,
            bool useStockInventoryControl,
            string storeID,
            string rootCategoryID
        );

        IList<Product> GetAllByTaxClassID(
            Culture culture,
            string taxClassID,
            string storeID
            );

        IList<Product> GetByIsQBExported(
            Culture culture,
            bool isQBExported,
            string rootCategoryID,
            string storeID,
            string existList );

        IList<Product> GetAllQBProduct(
            Culture culture,
            string rootCategoryID,
            string storeID
            );

        IList<TableSchemaItem> GetTableSchema();

        IList<Product> SearchProduct(
            Culture culture,
            string categoryID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID
            );

        bool HasProductInDiscountGroup( string discountgroupID );

        IList<Product> QuickSearch(
            Culture culture,
            string searchTerm,
            string[] searchFieldList,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string searchType
            );

        IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string searchType
            );

        IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string departmentIDs,
            string manufacturerID,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            string rootDepartmentID,
            string searchType
            );

        IList<Product> AdvancedSearch(
            Culture culture,
            string categoryIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootCategoryID,
            bool isIncludeProductKit,
            string searchType
        );

        IList<Product> AdvancedSearchDepartment(
            Culture culture,
            string departmentIDs,
            string sortBy,
            string searchTerm,
            string price1,
            string price2,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            string rootDepartmentID,
            string searchType
            );

        DataTable GetStockOptionLine( Culture culture, string[] optionGroupIDs );

        void UpdateSortOrder( string categoryID, string[] sortedProductIDs );
        void UpdateSortOrderDepartment( string departmentID, string[] sortedProductIDs );

        void UpdateQBData( string productID, string qbListID, bool isQBExported );
        void SetDefaultQBData( string qbListID );

        int CountQuickSearchResult(
            Culture culture,
            string searchTerm,
            string[] searchFieldList,
            string sortBy,
            string storeID,
            string rootCategoryID,
            string searchType
            );

        Product GetOneLargeProduct( Culture culture, string productID, string storeID );
        Product CreateLargeProductItem( Product product, ProductLocale locale, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption );
        Product UpdateLargeProductItem( Product product, ArrayList categoryVisibleIDList, bool processImage, bool isUseOption );
        Product GetOneBySku( Culture culture, string storeID, string sku );
        Product GetOneByUrlName( Culture culture, string storeID, string name );
        IList<Product> GetProductByName( Culture culture, string storeID, string name );

        bool IsUseCustomPrice( string productID );


        IList<Product> GetAllBySubscriptionLevelID(
            Culture culture,
            string subscriptionID,
            string storeID );

        IList<Product> GetAllProductKit( Culture culture );
        IList<Product> GetProductKitItems( Culture culture, string ProductKitGroupID, string sortBy );
        bool CheckIsProductKitByProductID( string productID );
        void IncreaseViewCount( string productID );

        Dictionary<string, string> GetMultipleCategoryProduct();
        Dictionary<string, string> GetMultipleDepartmentProduct();

        IList<Product> GetProductNewArrival(
            Culture culture,
            string storeID,
            int limitResult,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            string rootCategoryID,
            out int howManyItems );
        #endregion
    }
}
