using System;
using System.Collections.Generic;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.QuickBooks.Qbxml70;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for ProductVerifyRequestState
    /// </summary>
    public class ProductVerifyRequestState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new ProductVerifyResponseState( QBWCSession );
        }

        private void MoveToQueryState()
        {
            QBWCSession.CurrentState = new ProductQueryRequestState( QBWCSession );
        }

        private IList<Product> GetExportData()
        {
            string cultureID = DataAccessContext.CultureRepository.GetIDByName(
                DataAccessContext.Configurations.GetValue( "DefaultWebsiteLanguage" ) );
            Culture culture = DataAccessContext.CultureRepository.GetOne( cultureID );

            string storeID = getStoreID();
            string rootCategoryID = DataAccessContext.Configurations.GetValue( "0", "RootCategory", storeID );

            IList<Product> table = DataAccessContext.ProductRepository.GetByIsQBExported( 
                culture, true, rootCategoryID, storeID, String.Empty );

            return table;
        }

        private string getStoreID()
        {
            return DataAccessContext.Configurations.GetValue( "QBSelectStoreExport" );
        }

        private ItemNonInventoryQueryRqType CreateOneRequest( Product product )
        {
            ItemNonInventoryQueryRqType req = new ItemNonInventoryQueryRqType();
            req.Items = new object[] { product.QBListID };
            req.ItemsElementName = new ItemsChoiceType44[] { ItemsChoiceType44.ListID };
            return req;
        }

        private string CreateProductVerify( out bool xmlReady )
        {
            xmlReady = false;
            IList<Product> table = GetExportData();
            if (table.Count == 0)
            {
                MoveToQueryState();
                return String.Empty;
            }

            QBXML qbxml = CreateRequestQbxml();

            object[] requests = new object[table.Count];
            int index = 0;
            foreach (Product product in table)
            {
                requests[index] = CreateOneRequest( product );
                index++;
            }

            ((QBXMLMsgsRq) qbxml.Items[0]).Items = requests;

            xmlReady = true;

            string xml = CreateRequestXml( qbxml );
            return xml;
        }

        public ProductVerifyRequestState( QBWCSession session )
            : base( session )
        {
        }

        public override string OnSendRequestXml( out bool xmlReady )
        {
            MoveToNextState();

            return CreateProductVerify( out xmlReady );
        }
    }
}