﻿using System;
using Vevo.Shared.Utilities;
using Vevo.Domain;
using Vevo.Domain.Products;
using System.Collections.Generic;
namespace Vevo.WebUI.Products
{
    public class BaseFacetSearchControl : Vevo.WebUI.International.BaseLanguageUserControl
    {
        #region Private

        private bool IsCategoryList()
        {
            if (Request.Url.AbsoluteUri.ToLower().Contains( "catalog.aspx" ) && ConvertUtilities.ToInt32( CurrentCategoryID ) > 0)
                return true;
            else
                return false;
        }

        #endregion

        public string MinPrice
        {
            get
            {
                if (Request.QueryString["MinPrice"] == null)
                    return String.Empty;
                else
                    return Request.QueryString["MinPrice"];
            }
        }

        public string MaxPrice
        {
            get
            {
                if (Request.QueryString["MaxPrice"] == null)
                    return String.Empty;
                else
                    return Request.QueryString["MaxPrice"];
            }
        }

        public string CurrentCategoryID
        {
            get
            {
                string id = Request.QueryString["CategoryID"];
                if (!String.IsNullOrEmpty( id ))
                {
                    return id;
                }
                else
                {
                    Category category = DataAccessContext.CategoryRepository.GetOneByUrlName(
                        StoreContext.Culture, CurrentCategoryName );
                    return category.CategoryID;
                }
            }
        }

        public string CurrentCategoryName
        {
            get
            {
                if (Request.QueryString["CategoryName"] == null)
                    return String.Empty;
                else
                    return Request.QueryString["CategoryName"];
            }
        }

        public int PriceStep
        {
            get
            {
                return ConvertUtilities.ToInt32( DataAccessContext.Configurations.GetValue( "PriceNavigationStep", StoreContext.CurrentStore ) );
            }
        }

        public int MaxLevel
        {
            get
            {
                return ConvertUtilities.ToInt32( DataAccessContext.Configurations.GetValue( "MaximunInterval", StoreContext.CurrentStore ) );
            }
        }

        public Category CurrentCategory
        {
            get
            {
                return DataAccessContext.CategoryRepository.GetOne( StoreContext.Culture, CurrentCategoryID );
            }
        }

        public string[] GetCategoryListwithLeaf()
        {
            return GetCategoryListwithLeaf( CurrentCategoryID );
        }

        public string[] GetCategoryListwithLeaf( string categoryID )
        {
            IList<string> catlist = new List<string>();
            IList<string> categoryids = DataAccessContext.CategoryRepository.GetLeafFromCategoryID( categoryID, catlist );

            List<string> categoryCollection = new List<string>();
            foreach (string categoryItem in categoryids)
            {
                categoryCollection.Add( categoryItem );
            }

            return categoryCollection.ToArray();
        }

        public string[] GetDepartmentListWithLeaf()
        {
            string departmentID = "";

            if (!String.IsNullOrEmpty( Request.QueryString["dep"] ))
            {
                departmentID = Request.QueryString["dep"];
            }

            return GetDepartmentListWithLeaf( departmentID );
        }

        public string GetManufacturerID()
        {
            string manufacturerID = String.Empty;
            if (!String.IsNullOrEmpty( Request.QueryString["manu"] ))
            {
                manufacturerID = Request.QueryString["manu"];
            }
            return manufacturerID;
        }
        public string[] GetDepartmentListWithLeaf( string departmentID )
        {
            IList<string> departmentIDs = new List<string>();
            if (!String.IsNullOrEmpty( departmentID ))
            {
            IList<string> depList = new List<string>();
                departmentIDs = DataAccessContext.DepartmentRepository.GetLeafFromDepartmentID( departmentID, depList );
            }
            List<string> departmentCollection = new List<string>();
            foreach (string departmentItem in departmentIDs)
            {
                departmentCollection.Add( departmentItem );
            }

            return departmentCollection.ToArray();
        }

        public bool IsFacetedVisible()
        {
            if (IsCategoryList())
            {
                return DataAccessContext.Configurations.GetBoolValue( "FacetedSearchEnabled", StoreContext.CurrentStore ) && CurrentCategory.IsAnchor;
            }
            else
                return false;
        }

        public IList<string> GetAllSpecKey()
        {
            string[] allKey = Request.QueryString.AllKeys;

            string query = String.Empty;
            IList<string> specKeyList = new List<string>();
            for (int i = 0; i < allKey.Length; i++)
            {
                if ( !allKey[ i ].ToLower().Equals( "categoryname" ) && !allKey[ i ].ToLower().Equals( "categoryid" ) )
                {
                    if (allKey[i].ToLower().Equals( "minprice" ) ||
                        allKey[i].ToLower().Equals( "maxprice" ) ||
                        allKey[i].ToLower().Equals( "cat" ) ||
                        allKey[i].ToLower().Equals( "dep" ) ||
                        allKey[i].ToLower().Equals( "manu" ))
                    {
                        continue;
                    }

                    specKeyList.Add( allKey[i] );
                }
            }

            return specKeyList;
        }

        public IList<SpecificationItemValue> GetSpecItemValueList( IList<string> specKeyList, SpecificationItemValue currentSpecItemValue )
        {
            IList<SpecificationItemValue> specItemValueList = new List<SpecificationItemValue>();
            foreach (string specKey in specKeyList)
            {
                string specValue = Request.QueryString[specKey];

                SpecificationItem specItem = DataAccessContext.SpecificationItemRepository.GetOneByName( StoreContext.Culture, specKey );

                SpecificationItemValue specItemValue = DataAccessContext.SpecificationItemValueRepository.GetOneBySpecItemIDAndValue( StoreContext.Culture, specItem.SpecificationItemID, specValue );
                specItemValueList.Add( specItemValue );
            }

            specItemValueList.Add( currentSpecItemValue );

            return specItemValueList;
        }

        public IList<SpecificationItemValue> GetSpecItemValueList( IList<string> specKeyList )
        {
            IList<SpecificationItemValue> specItemValueList = new List<SpecificationItemValue>();
            foreach (string specKey in specKeyList)
            {
                string specValue = Request.QueryString[specKey];

                SpecificationItem specItem = DataAccessContext.SpecificationItemRepository.GetOneByName( StoreContext.Culture, specKey );

                SpecificationItemValue specItemValue = DataAccessContext.SpecificationItemValueRepository.GetOneBySpecItemIDAndValue( StoreContext.Culture, specItem.SpecificationItemID, specValue );
                specItemValueList.Add( specItemValue );
            }

            return specItemValueList;
        }
    }
}
