using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Web.UI.WebControls;
using Vevo;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.SystemServices;
using System.Web;

namespace Vevo.WebUI.Products
{
    public class OptionUploadWebControlService
    {
        #region Private

        private bool _uploadRequired;
        private string _webAppLocalPath;

        private bool _hasFile;
        private string _fileName;
        private HttpPostedFile _postedFile;

        private bool IsFileExtensionUploadable( string fileName )
        {
            string chkExt = Path.GetExtension( fileName );
            string[] allowExt = DataAccessContext.Configurations.GetValueList( "UploadExtension" );
            for (int i = 0; i < allowExt.Length; i++)
            {
                if (String.Compare( chkExt, "." + allowExt[i], true ) == 0)
                    return true;
            }

            return false;
        }

        private TempUploadedFile UploadFile()
        {
            if (!_hasFile)
                return null;

            TempUploadedFile tempUploadedFile = new TempUploadedFile(
                new FileManager(),
                SystemConst.OptionFileTempPath,
                SystemConst.OptionFileUpload,
                _fileName );

            if (_postedFile != null)
                _postedFile.SaveAs( tempUploadedFile.GetTempFileLocalPath() );

            return tempUploadedFile;
        }

        #endregion


        #region Constructors

        public OptionUploadWebControlService( bool uploadRequired, FileUpload fileUpload, string webAppLocalPath )
        {
            _uploadRequired = uploadRequired;
            _hasFile = fileUpload.HasFile;
            _fileName = fileUpload.FileName;
            _postedFile = fileUpload.PostedFile;
            _webAppLocalPath = webAppLocalPath;
        }

        public OptionUploadWebControlService( bool uploadReqired, string fileNameUploaded, string webAppLocalPath )
        {
            _uploadRequired = uploadReqired;
            _hasFile = true;
            _fileName = fileNameUploaded;
            _webAppLocalPath = webAppLocalPath;
        }

        #endregion


        #region Public Methods

        public bool ValidateInput( out string errorMessage )
        {
            errorMessage = String.Empty;

            if (!_hasFile)
            {
                if (_uploadRequired)
                {
                    errorMessage = "[$OptionUploadRequired]";
                    return false;
                }
                else
                {
                    // File upload is not required. The user can skip this upload.
                    return true;
                }
            }

            try
            {
                if (_postedFile != null)
                {
                    if (_postedFile.ContentLength
                        <= DataAccessContext.Configurations.GetIntValue( "UploadSize" ) * SystemConst.UploadSizeConfigFactor)
                    {
                        if (IsFileExtensionUploadable( _fileName ))
                        {
                            return true;
                        }
                        else
                        {
                            errorMessage = "[$OptionUploadExtension]";
                        }
                    }
                    else
                    {
                        errorMessage = "[$OptionUploadMaxLimit]";
                    }
                }
                else
                    return true;
            }
            catch
            {
                errorMessage = "[$OptionUploadInvalid]";
            }

            return false;
        }

        public OptionItemValue CreateOptionItemValue( string optionItemID, bool useStock )
        {
            if (!_hasFile)
                return null;

            OptionItem optionItem = DataAccessContext.OptionItemRepository.GetOne( Culture.Null, optionItemID );

            return new OptionItemValue(
                optionItem,
                OptionGroup.OptionGroupType.Upload,
                _fileName,
                useStock,
                UploadFile() );
        }

        public OptionItemValue CreateOptionItemValue( string optionItemID, bool useStock,TempUploadedFile uploadedFile )
        {
            if (!_hasFile)
                return null;

            OptionItem optionItem = DataAccessContext.OptionItemRepository.GetOne( Culture.Null, optionItemID );

            return new OptionItemValue(
                optionItem,
                OptionGroup.OptionGroupType.Upload,
                _fileName,
                useStock,
                uploadedFile );
        }

        #endregion

    }
}
